/*
 *  Addable - the interface to the record format of PDBFile
 *  Copyright (C) 2000  James Thrasher
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *
 *  email: jjt@gjt.org
 */

package org.gjt.jjt.pdbfile;

/**
 * interface to anything we can add (in record form) to a PDB file.  This includes
 * the AppInfoBlock and SortInfoBlock, though they are only used once.
 *
 * @author James Thrasher
 * @version .1
 */
public interface Addable {

    /*
     * attributes for a record.
     */
    public static final byte SECRET_RECORD_BIT = (byte) 16;
    public static final byte BUSY_BIT = (byte) 32;
    public static final byte DIRTY_BIT = (byte) 64;
    public static final byte DEL_ON_HSYNC_BIT = (byte) 128;

    /**
     * get the size of this particular Addable.
     *
     * @return the size (int) in bytes of this Addable
     */
    public int size();

    /**
     * set the record attributes of this Addable.
     *
     * @param attr the bit array (8 bits) that are the attributes to be set
     */
    public void setAttributes(byte attr);

    /**
     * get the record attributes of this Addable.
     *
     * @return the bit array (8 bits (1 byte)) that are the attributes of this Addable.
     */
    public byte getAttributes();

    /**
     * return a byte array that is the raw data that makes up this record.  Used in
     * writing the record to file.
     *
     * @return the byte array to be written to file.
     */
    public byte[] getBytes();

    /**
     * given a byte array of raw data, parse it into the real data.  Used in reading
     * a record from a file.
     *
     * @param data[] the byte array to parse.
     */
    public void setBytes(byte data[]);
}
