/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:58 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv;

import java.util.*;
import java.io.*;

import org.zaval.util.io.*;

public class AllChannels implements Serializable {
    protected ArrayList sites;
    transient HashSet Pinclude;
    transient HashSet Pexclude;

    public AllChannels() {
        sites = new ArrayList();
    }

    public void setProperties(AProperties p) {
        if (p == null) return;
        if (Pinclude == null) Pinclude = new HashSet();
        if (Pexclude == null) Pexclude = new HashSet();

        StringTokenizer st;
        Pinclude.clear();
        st = new StringTokenizer(p.getProperty("list.include", ""), ";");
        while (st.hasMoreTokens()) Pinclude.add(st.nextToken());
        Pexclude.clear();
        st = new StringTokenizer(p.getProperty("list.exclude", ""), ";");
        while (st.hasMoreTokens()) Pexclude.add(st.nextToken());
    }

    public int size() {
        return sites.size();
    }

    public void add(OneSite s) {
        sites.add(s);
    }

    public OneSite elementAt(int i) {
        return (OneSite) sites.get(i);
    }

    public void removeElementAt(int i) {
        sites.remove(i);
    }

    public OneSite findcreate(String name) {
        OneSite res = find(name);
        if (res == null) {
            res = new OneSite(name);
            add(res);
        }
        return res;
    }

    public void organize() { // erase clean channels
        int i,j;
        OneChannel cp;
        OneSite s;

        for (i = 0; i < size(); i++) {
            s = elementAt(i);
            for (j = 0; j < s.size(); j++) {
                if (s.elementAt(j).size() == 0) {
                    s.removeElementAt(j);
                    j--;
                }
            }
        }
    }

    public OneSite find(String sn) {
        OneSite p;
        for (int i = 0; i < size(); i++) {
            p = elementAt(i);
            if (sn.equals(p.SiteName)) return p;
        }
        return null;
    }

    public void merge(AllChannels newlist, boolean allowoverride) {
        OneChannel newch,oldch;
        OneSite site,sitenew;
        int i,j;
        for (int k = 0; k < newlist.size(); k++) {
            sitenew = newlist.elementAt(k);
            site = findcreate(sitenew.SiteName);
            for (i = 0; i < sitenew.size(); i++) {
                newch = sitenew.elementAt(i);
                oldch = site.find(newch.ChannelName);
                if (oldch == null)
                    site.add(newch);
                else {
                    if (!allowoverride) { // override disable
                        // try to detect holes
                        boolean hole = false;

                        oldch.elementAt(oldch.size() - 1).tryCorrect(newch.begTime());
                        newch.elementAt(newch.size() - 1).tryCorrect(oldch.begTime());
                        if (oldch.endTime() <= newch.begTime()) hole = true;
                        if (newch.endTime() <= oldch.begTime()) hole = true;
                        for (j = 0; j < newch.size() - 1; j++) { // real holes
                            newch.elementAt(j).tryCorrect(oldch.begTime());
                            oldch.elementAt(oldch.size() - 1).tryCorrect(newch.elementAt(j + 1).getBegin());
                            if (newch.elementAt(j).getEnd() <= oldch.begTime() && newch.elementAt(j + 1).getBegin() >= oldch.endTime()) {
                                hole = true;
                                break;
                            }
                        }
                        for (j = 0; j < oldch.size() - 1; j++) { // real holes
                            oldch.elementAt(j).tryCorrect(newch.begTime());
                            newch.elementAt(newch.size() - 1).tryCorrect(oldch.elementAt(j + 1).getBegin());
                            if (oldch.elementAt(j).getEnd() <= newch.begTime() && oldch.elementAt(j + 1).getBegin() >= newch.endTime()) {
                                hole = true;
                                break;
                            }
                        }
                        // end trying
                        if (hole) { // allow merge
                            for (j = 0; j < newch.size(); j++)
                                oldch.add(newch.elementAt(j));
/*                        System.err.println("Merge ok "+newch.ChannelName);
                        System.err.println(" oldch.beg="+(new Date(oldch.begTime())).toString()+" "+oldch.begTime());
                        System.err.println(" oldch.end="+(new Date(oldch.endTime())).toString()+" "+oldch.endTime());
                        System.err.println(" newch.beg="+(new Date(newch.begTime())).toString()+" "+newch.begTime());
                        System.err.println(" newch.end="+(new Date(newch.endTime())).toString()+" "+newch.endTime());*/
/*                        for (j=0; j<newch.size(); j++) {
                            System.out.println("   "+(new Date(newch.elementAt(j).BegTime)).toString()+" "+newch.elementAt(j).ProgramName+"/"+newch.elementAt(j).last);
                        }
                        System.err.println(" after.beg="+(new Date(oldch.begTime())).toString()+" "+oldch.begTime());
                        System.err.println(" after.end="+(new Date(oldch.endTime())).toString()+" "+oldch.endTime());*/
                            oldch.sort();
                        } else {
                            System.err.println("Warning: can't merge channel " + newch.ChannelName);
                            System.err.println(" oldch.beg=" + (new Date(oldch.begTime())).toString());
                            System.err.println(" oldch.end=" + (new Date(oldch.endTime())).toString());
                            System.err.println(" newch.beg=" + (new Date(newch.begTime())).toString());
                            System.err.println(" newch.end=" + (new Date(newch.endTime())).toString());
/*                        for (j=0; j<newch.size(); j++)
                            System.out.println("   "+(new Date(newch.elementAt(j).BegTime)).toString()+" "+newch.elementAt(j).ProgramName+"/"+newch.elementAt(j).last);
                        System.out.println("==================================================");
                        for (j=0; j<oldch.size(); j++)
                            System.out.println("   "+(new Date(oldch.elementAt(j).BegTime)).toString()+" "+oldch.elementAt(j).ProgramName+"/"+oldch.elementAt(j).last);
                            */
                        }
                    } else { // override enable
                        oldch.deleteInterval(newch.begTime(), newch.endTime());
                        for (j = 0; j < newch.size(); j++)
                            oldch.add(newch.elementAt(j));
                        oldch.sort();
                    }
                }
            }
        }
    }
}
