/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:58 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv;

import java.util.regex.*;
import java.util.*;
import java.io.*;

public class OneChannel implements Comparator, Serializable {
    public String ChannelName = null;
    protected ArrayList data;

    public OneChannel(String name) {
        data = new ArrayList();
        ChannelName = name;
    }

    public int size() {
        return data.size();
    }

    public void add(Prog p) {
        data.add(p);
    }

    public Prog elementAt(int i) {
        return (Prog) data.get(i);
    }

    public void removeElementAt(int i) {
        data.remove(i);
    }

    public void clear() {
        data.clear();
    }

    public Object[] toArray() {
        return data.toArray();
    }

    public void sort() {
        Object[] a = toArray();
        Arrays.sort(a, this);
        clear();
        for (int i = 0; i < a.length; i++) add((Prog) a[i]);
    }

    public int compare(Object o1, Object o2) {
        return (int) (((Prog) o1).BegTime - ((Prog) o2).BegTime);
    }

    public long begTime() {
        if (size() > 0)
            return elementAt(0).getBegin();
        else
            return -1;
    }

    public long endTime() {
        if (size() > 0)
            return elementAt(size() - 1).getEnd();
        else
            return -1;
    }

    public void deleteInterval(long dfrom, long dto) {
        for (int i = 0; i < size(); i++) {
            Prog p = elementAt(i);
            if ((p.BegTime >= dfrom && p.BegTime < dto) || (p.getEnd() >= dfrom && p.getEnd() < dto)) {
                removeElementAt(i);
                i--;
            }
        }
    }

    public void purge(long firstdate) {
        Prog p;
        for (int i = 0; i < size(); i++) {
            p = elementAt(i);
            if (p.getEnd() <= firstdate) {
                removeElementAt(i);
                i--;
            }
        }
    }

    public Prog find(long beg) {
        for (int i = 0; i < size(); i++)
            if (elementAt(i).BegTime == beg) return elementAt(i);
        return null;
    }

    public boolean equals(Object o) {
        if (o == null) return false;
        OneChannel other = (OneChannel) o;
        Prog pmy,po;
        if (size() != other.size()) return false;
        for (int i = 0; i < size(); i++) {
            pmy = elementAt(i);
            po = other.find(pmy.BegTime);
            if (!pmy.equals(po)) return false;
        }
        return true;
    }
}
