/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:58 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv;

import java.util.regex.*;
import java.util.*;
import java.io.*;

import org.zaval.util.format.*;

public class ParseOneChannel extends OneChannel {
    transient protected Pattern reProgram,reProgramLen,reProgramEnd;

    public ParseOneChannel(String name) {
        super(name);
        try {
            reProgram = Pattern.compile("^(\\d{1,2}[\\.|:]\\d{2})([ |,]*)(.+)$", Pattern.CASE_INSENSITIVE);
            reProgramLen = Pattern.compile("^(\\d{1,2}[\\.|:]\\d{2})-(\\d{1,2}[\\.|:]\\d{2})\\s+(.+)$", Pattern.CASE_INSENSITIVE);
            reProgramEnd = Pattern.compile("^(.+)\\(до\\s+(\\d{1,2}[\\.|\\s|:]\\d{2})\\)$", Pattern.CASE_INSENSITIVE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE in ChannelProg : " + e.getMessage());
        }
    }

    public ParseProg findByTime(long time) {
        for (int i = 0; i < size(); i++) {
            ParseProg p = elementAtP(i);
            if (p.BegTime == time) return p;
        }
        return null;
    }

    public ParseProg elementAtP(int i) {
        return (ParseProg) elementAt(i);
    }

    public void checktime(long time) throws PTimeException {
        if (time == -1) throw new PTimeException("Bad time format !");
    }

    public boolean canParse(String str) {
        return reProgram.matcher(str).find();
    }

    public void parse(String str, long basedate, TimeZone htz)
            throws ProgException, PTimeException {
        Matcher m;
        long begtime,endtime;
        Vector timelist = new Vector();

        String p = str;
        String sp;

        if ((m = reProgramEnd.matcher(p)).find()) {
            p = m.group(1);
            checktime(endtime = TimeEngine.parseTime(m.group(2), htz, basedate));
            endtime += basedate;
        } else
            endtime = 0;

        if ((m = reProgramLen.matcher(p)).find()) {
            checktime(begtime = TimeEngine.parseTime(m.group(1), htz, basedate));
            checktime(endtime = TimeEngine.parseTime(m.group(2), htz, basedate));
            if (endtime < begtime)
                endtime += 24L * 60 * 60 * 1000;
            begtime += basedate;
            endtime += basedate;
            add(new ParseProg(true, begtime, (short) (endtime - begtime) / 1000 / 60, m.group(3)));
            return;
        }

        while ((m = reProgram.matcher(p)).find()) {
            checktime(begtime = TimeEngine.parseTime(m.group(1), htz, basedate));
            begtime += basedate;
            timelist.add(new Long(begtime));
            p = m.group(3);
            sp = m.group(2);
            if (sp.length() == 0) break;
            if (sp.charAt(0) == ' ')
                break;
            else if (sp.charAt(0) == ',')
                continue;
            else
                throw new ProgException("Bad line format - bad time list separator !");
        }

        if (timelist.size() == 0) throw new ProgException("Bad line format - time not found ! (" + str + ")");
        for (int i = 0; i < timelist.size(); i++)
            add(new ParseProg(i == 0, ((Long) timelist.elementAt(i)).longValue(), (short) 0, p));
        if (endtime != 0) (elementAtP(size() - 1)).setTimeEnd(endtime);
    }

    public void addAnnouncement(String str) {
        int i;
        ParseProg curr;
        for (i = size() - 1; i >= 0; i--) {
            curr = elementAtP(i);
            if (curr.First) {
                curr.addAnnouncement(str);
                return;
            }
        }
    }
/*    public void sort() {
        int i,lastH;
        Prog p;
        // make raw list
        lastH=0;
        for (i=0; i<size(); i++) {
            p=elementAt(i);
            if (!p.First) continue;
            while (p.BegTime.h<lastH) {
                p.BegTime.h+=24;
            }
            lastH=p.BegTime.h;
        }
        for (i=0; i<size(); i++) {
            p=elementAt(i);
            if (p.First) {lastH=p.BegTime.h;continue;}
            while (p.BegTime.h<lastH) {
                p.BegTime.h+=24;
            }
            lastH=p.BegTime.h;
        }
        Object[] a=toArray();
        Arrays.sort(a,this);
        clear();
        for (i=0; i<a.length; i++) add((Prog)a[i]);
    }*/
    public void sort24() {
        int i;
        Prog p;
        if (size() == 0) return;
        long firsttime = elementAtP(0).BegTime;
        for (i = 1; i < size(); i++) {
            p = elementAt(i);
            if (p.BegTime < firsttime)
                p.BegTime += 24L * 60 * 60 * 1000;
        }
        super.sort();
    }

    public void calcEndTime(short deflen) {
        int i;
        ParseProg p,p2;
        for (i = 0; i < size(); i++) {
            p = elementAtP(i);
            if (p.LengthTime == 0) {
                if (i == size() - 1) {
                    p.LengthTime = deflen;
                    p.last = true;
                } else {
                    p2 = elementAtP(i + 1);
                    p.LengthTime = (short) ((p2.BegTime - p.BegTime) / 1000L / 60);
                }
            }
        }
    }
}
