/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:59 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.util.jar.*;
import java.text.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.io.*;
import org.zaval.util.format.*;

public class TVProgram {
    public static AProperties tvp,tvpatch;
    public static AllChannels list;
    static int Pcarg;
    static String Pargs[];
    public static String DataDir = "data";
    public static int numPage = 100;

    public static String getNextArg() {
        if (Pcarg >= Pargs.length) return null;
        Pcarg++;
        return Pargs[Pcarg - 1];
    }

    public static void loadProperties() {
        tvp = new AProperties();
        tvpatch = new AProperties();
        list = null;
        try {
            tvp.load("atv.ini");
            tvpatch.loadPatch("atv.patch");
        } catch (FileNotFoundException e) {
        } catch (UnsupportedEncodingException e) {
            System.err.println("Unsupported encoding in config : " + e.getMessage());
            System.exit(1);
        } catch (IOException e) {
            System.err.println("I/O error reading confg : " + e.getMessage());
            System.exit(1);
        }
    }

    public static void main(String args[]) {
        Pcarg = 0;
        Pargs = args;

        // hack for output data in fucking windows
        if (Locale.getDefault().getLanguage().equals("ru") &&
                System.getProperty("os.name", "").startsWith("Windows")) {
            try {
                System.setOut(new PrintStream((OutputStream) System.out, true, "Cp866"));
                System.setErr(new PrintStream((OutputStream) System.err, true, "Cp866"));
            } catch (UnsupportedEncodingException e) {
            }
        }
        // end hack

        String cmd = getNextArg();
        if (cmd == null) doListCommands();

        loadProperties();

        String tz = tvp.getProperty("local.TimeZone");
        if (tz == null) {
            TimeEngine.localTZ = TimeZone.getDefault();
        } else {
            TimeEngine.localTZ = TimeZone.getTimeZone(tz);
        }

        if (cmd.startsWith("--datadir=")) {
            DataDir = cmd.substring(10);
            cmd = getNextArg();
        }

        if (cmd.equals("parse")) {
            String module = getNextArg();
            if (module == null)
                doDispList();
            else {
                list = new AllChannels();
                if (doParse(module)) {
                    saveData(true);
                }
            }
        } else if (cmd.equals("purge")) {
            String time = getNextArg();
            int hr;
            if (time == null)
                hr = 24;
            else
                hr = Integer.parseInt(time);
            if (hr < 0) hr = 24;
            loadData(false);
            doPurge(hr);
            saveData(false);
        } else if (cmd.equals("load")) {
            String module = getNextArg();
            if (module == null)
                doDispList();
            else {
                list = new AllChannels();
                doLoad(module);
                saveData(true);
            }
        } else if (cmd.equals("save")) {
            String module = getNextArg();
            if (module == null)
                doDispList();
            else {
                loadData(false);
                String fn = getNextArg();
                doSave(module, fn);
            }
        } else if (cmd.equals("list")) {
            loadData(false);
            doList(false, 0, 0);
        } else if (cmd.equals("fulllist")) {
            loadData(false);
            doList(true, 0, 0);
        } else if (cmd.equals("now")) {
            loadData(false);
            String hc = getNextArg();
            int h = 0;
            if (hc != null) {
                try {
                    h = Integer.parseInt(hc);
                } catch (NumberFormatException e) {
                    System.err.println("Number format exception when parse command line : " + e.getMessage());
                    System.exit(1);
                }
            }
            doList(true, (new Date()).getTime(), (new Date()).getTime() + h * 60 * 60 * 1000);
        } else if (cmd.equals("system")) {
            doSystem();
        } else
            doListCommands();
    }

    public static void loadData(boolean AllowNotFound) {
        list = new AllChannels();

        tvFinternal atv = new tvFinternal();
        atv.setProperties(tvp);
        try {
            AllChannels newa = new AllChannels();
            atv.loadFromFile(DataDir + File.separator, newa);
            list.merge(newa, false);
        } catch (FileNotFoundException e) {
            if (!AllowNotFound) {
                System.err.println("Files data/*.xml not found : " + e.getMessage());
                e.printStackTrace();
                System.exit(2);
            }
        } catch (IOException e) {
            System.err.println("I/O error when read data/*.xml : " + e.getMessage());
            e.printStackTrace();
            System.exit(2);
        }
        list.setProperties(tvp);
    }

    public static void saveData(boolean needOrganize) {
        if (needOrganize)
            list.organize();
        try {
            tvFinternal atv = new tvFinternal();
            atv.setProperties(tvp);
            atv.saveToFile(DataDir + File.separator, list);
        } catch (FileNotFoundException e) {
            System.err.println("Can't create file data/*.xml : " + e.getMessage());
            System.exit(2);
        } catch (IOException e) {
            System.err.println("I/O error when write data/*.xml : " + e.getMessage());
            System.exit(2);
        }
    }

    public static void doSystem() {
        Properties prop = System.getProperties();
        Enumeration keys = prop.keys();
        while (keys.hasMoreElements()) {
            String key = String.valueOf(keys.nextElement());
            System.out.println("" + key + "==>" + prop.getProperty(key));
        }
        System.out.println("==============================================");
        String tzs[] = TimeZone.getAvailableIDs();
        for (int i = 0; i < tzs.length; i++) {
            System.out.println(tzs[i]);
        }
        TimeZone tz = TimeZone.getTimeZone("Europe/Minsk");
        System.out.println("==============================================");
        System.out.println("Current tzoffset=" + TimeEngine.getOffset((new Date()).getTime(), tz));
        System.out.println("os.name=" + System.getProperty("os.name") + " os.arch=" + System.getProperty("os.arch"));
        System.out.println("Locale.getLanguage=" + Locale.getDefault().getLanguage());
        System.out.println("path sep=" + File.separator);
    }

    public static void doList(boolean full, long dfrom, long dto) {
        OneChannel cp;
        OneSite s;
        if (!full) {
            for (int i = 0; i < list.size(); i++) {
                s = list.elementAt(i);
                System.out.println("  site : " + s.SiteName);
                for (int j = 0; j < s.size(); j++)
                    System.out.println(s.elementAt(j).ChannelName);
            }
        } else {
            Prog p;
            DateFormat outdate = DateFormat.getDateInstance(DateFormat.LONG);
            DateFormat outtime = DateFormat.getTimeInstance(DateFormat.SHORT);
            outdate.setTimeZone(TimeEngine.localTZ);
            outtime.setTimeZone(TimeEngine.localTZ);
            for (int i = 0; i < list.size(); i++) {
                s = list.elementAt(i);
                System.out.println("=========== site : " + s.SiteName);
                for (int k = 0; k < s.size(); k++) {
                    cp = s.elementAt(k);
                    String day = "";
                    Date d;
                    if (full) {
                        for (int j = 0; j < cp.size(); j++) {
                            p = cp.elementAt(j);
                            if (dfrom > 0) {
                                if (p.BegTime >= dto) continue;
                                if (p.getEnd() <= dfrom) continue;
                            }
                            d = new Date(p.BegTime);
                            if (!outdate.format(d).equals(day)) {
                                System.out.println(cp.ChannelName + " " + outdate.format(d));
                                day = outdate.format(d);
                            }
                            System.out.println("  " + outtime.format(new Date(p.BegTime)) + "/" + p.LengthTime + " " + p.ProgramName);
                            if (p.Announcement != null) System.out.println("A:" + p.Announcement);
                        }
                    }
                }
            }
        }
    }

    public static void doListCommands() {
        System.err.println("Usage : java -jar atv.jar <command>");
        System.err.println("   commands:");
        System.err.println("     parse [module name] [--charset=<charset>] [filename1]...[filenameN] - parse TV-program from WWW(or from files with specified charset)");
        System.err.println("     load [module name] [filename1]..[filenameN] - load TV-program from file");
        System.err.println("     save [module name] [filename] - save TV-program to file");
        System.err.println("     list                 - view channels list");
        System.err.println("     fulllist             - view full program list");
        System.err.println("     now [hours]          - view current(and later) programs");
        System.err.println("     purge [hours]        - purge old data(older than 1 days default)");
        System.err.println("");
        System.err.println(" Detailed description see at http://www.zaval.org/peojects/atv/");
        System.exit(1);
    }

    public static boolean doParse(String module) {
        boolean needsave = false;
        tvSuperP tvld = null;
        try {
            Class ld = Class.forName("org.zaval.atv.loader.tvP" + module);
            tvld = (tvSuperP) ld.newInstance();
        } catch (ClassNotFoundException e) {
            System.err.println("Error start module " + module + " : " + e.getMessage());
            System.exit(1);
        } catch (IllegalAccessException e) {
            System.err.println("Error access module " + module + " : " + e.getMessage());
            System.exit(1);
        } catch (InstantiationException e) {
            System.err.println("Error instant module " + module + " : " + e.getMessage());
            System.exit(1);
        }

        Transport tr = new Transport(tvpatch);
        String proxy_host = tvp.getProperty("local.proxy_host");
        int proxy_port = 0;
        try {
            proxy_port = Integer.parseInt(tvp.getProperty("local.proxy_port", "8080"));
        } catch (NumberFormatException e) {
            System.err.println("Number format exception when parse local.proxy_port : " + e.getMessage());
            System.exit(1);
        }
        String proxy_auth = tvp.getProperty("local.proxy_auth");
        if (proxy_host != null) tr.setProxy(proxy_host, proxy_port, proxy_auth);
        tvld.setProperties(tvp, tr);

        try {
            ParseAllChannels newa = new ParseAllChannels();

            boolean wasparse = false;
            String fn = null,charset = null;
            while (true) {
                fn = getNextArg();
                if (wasparse && fn == null) break;
                if (fn != null)
                    if (fn.equals("--list")) {
                        tvld.getList(tr);
                        break;
                    }
                if (fn != null)
                    if (fn.startsWith("--charset=")) {
                        charset = fn.substring(10);
                        continue;
                    }
                if (fn == null) {
                    tvld.loadFromWWW(tr, newa);
                    needsave = true;
                } else {
                    tr.setCharset(charset);
                    tr.OpenFILE(fn);
                    try {
                        tvld.loadFromFile(tr, newa);
                        needsave = true;
                    } finally {
                        tr.Close();
                    }
                }
                wasparse = true;
            }
            tr.report();
            if (needsave) {
                if (newa.size() > 0) {
                    newa.organize();
                    list.merge(newa, tvp.getProperty("list.override", "no").equals("yes"));
                } else {
                    System.err.println("Error load data from '" + module + "' module !!!");
                    System.exit(3);
                }
            }
        } catch (UnsupportedEncodingException e) {
            System.err.println("Unsupported encoding in input data file : " + e.getMessage());
            System.exit(2);
        } catch (IOException e) {
            System.err.println("I/O error in module " + module + " : " + e.getMessage());
            e.printStackTrace();
            System.exit(2);
        } catch (ProgException e) {
            System.err.println("Parse error in line # " + tr.getCurrentLineNum() + " : " + e.getMessage());
            System.err.println("Error line : =" + tr.oldline + "=");
            System.exit(2);
        } catch (PTimeException e) {
            System.err.println("Parse time error in line # " + tr.getCurrentLineNum() + " : " + e.getMessage());
            System.err.println("Error line : =" + tr.oldline + "=");
            System.exit(2);
        }
        return needsave;
    }

    public static void doDispList() {
        String fn = TVProgram.class.getResource("TVProgram.class").getFile();
        int bc = fn.indexOf("!");
        if (bc == -1) return;
        fn = fn.substring(5, bc);
        System.out.println(fn);
        try {
            JarFile jf = new JarFile(fn);
            Pattern reClasses;
            Matcher m;
            Enumeration en;
            JarEntry je;

            reClasses = Pattern.compile("^org/zaval/atv/loader/tvP(\\w+)\\.class$");
            en = jf.entries();
            System.err.println("  supported parser modules:");
            while (en.hasMoreElements()) {
                je = (JarEntry) en.nextElement();
                if ((m = reClasses.matcher(je.getName())).find())
                    System.err.println("       " + m.group(1));
            }
            reClasses = Pattern.compile("^org/zaval/atv/loader/tvF(\\w+)\\.class$");
            en = jf.entries();
            System.err.println("  supported format modules:");
            while (en.hasMoreElements()) {
                je = (JarEntry) en.nextElement();
                if ((m = reClasses.matcher(je.getName())).find())
                    System.err.println("       " + m.group(1));
            }
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE in TVProgramLoader : " + e.getMessage());
            System.exit(1);
        } catch (IOException e) {
            System.err.println("Error read atv.jar for list modules : " + e.getMessage());
            System.exit(1);
        }
    }

    public static void doPurge(int hr) {
        long ondate = (new Date()).getTime() - hr * 60L * 60 * 1000;
        System.out.println("purge data older " + hr + " hours");
        for (int i = 0; i < list.size(); i++)
            for (int j = 0; j < list.elementAt(i).size(); j++)
                list.elementAt(i).elementAt(j).purge(ondate);
    }

    public static void doSave(String module, String filename) {
        tvSuperF f = null;
        try {
            Class ld = Class.forName("org.zaval.atv.loader.tvF" + module);
            f = (tvSuperF) ld.newInstance();
        } catch (ClassNotFoundException e) {
            System.err.println("Error start module " + module + " : " + e.getMessage());
            System.exit(1);
        } catch (IllegalAccessException e) {
            System.err.println("Error access module " + module + " : " + e.getMessage());
            System.exit(1);
        } catch (InstantiationException e) {
            System.err.println("Error instant module " + module + " : " + e.getMessage());
            System.exit(1);
        }

        f.setProperties(tvp);
        try {
            f.saveToFile(filename, list);
        } catch (IOException e) {
            System.err.println("Error(module " + module + ") writing " + filename + " : " + e.getMessage());
            e.printStackTrace();
            System.exit(3);
        } catch (ProgException e) {
            System.err.println("Error(module " + module + ") writing " + filename + " : " + e.getMessage());
            System.exit(3);
        } catch (PTimeException e) {
            System.err.println("Error(module " + module + ") writing " + filename + " : " + e.getMessage());
            System.exit(3);
        }
    }

    public static void doLoad(String module) {
        tvSuperF f = null;
        try {
            Class ld = Class.forName("org.zaval.atv.loader.tvF" + module);
            f = (tvSuperF) ld.newInstance();
        } catch (ClassNotFoundException e) {
            System.err.println("Error start module " + module + " : " + e.getMessage());
            System.exit(1);
        } catch (IllegalAccessException e) {
            System.err.println("Error access module " + module + " : " + e.getMessage());
            System.exit(1);
        } catch (InstantiationException e) {
            System.err.println("Error instant module " + module + " : " + e.getMessage());
            System.exit(1);
        }

        f.setProperties(tvp);
        String fn = null;
        try {
            AllChannels newa = new AllChannels();
            while ((fn = getNextArg()) != null) {
                f.loadFromFile(fn, newa);
            }
            list.merge(newa, false);
        } catch (IOException e) {
            System.err.println("Error(module " + module + ") reading " + fn + " : " + e.getMessage());
            e.printStackTrace();
            System.exit(3);
        } catch (ProgException e) {
            System.err.println("Error(module " + module + ") reading " + fn + " : " + e.getMessage());
            System.exit(3);
        } catch (PTimeException e) {
            System.err.println("Error(module " + module + ") reading " + fn + " : " + e.getMessage());
            System.exit(3);
        }
    }
}
