/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.4 $
 *     $Date: 2004/03/31 10:44:11 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.net.*;
import java.io.*;
import java.util.*;

import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class Transport {
    public static final String HEADER_PROXY_AUTH = "Proxy-Authorization";
    public static final String HEADER_ACCEPT_LANGUAGE = "Accept-Language";
    public static final String HEADER_ACCEPT_CHARSET = "Accept-Charset";
    public static final String HEADER_REFERER = "Referer";
    public static final String HEADER_COOKIE = "Cookie";

    protected BufferedReader in = null;
    protected int CurrentLineNum = 0;
    protected boolean isEOF = true;
    protected String charset = null;
    protected Vector buffer = null;
    protected int page = 0;
    protected long downloaded = 0;
    public String oldline = null;
    protected AProperties patch;

    protected Map headers = null;
    protected Map cookies = null;


    public Transport(AProperties p) {
        patch = p;
    }

    protected String addcount(String str) {
        if (str != null)
            downloaded += str.length() + 1;
        return str;
    }

    public void report() {
        System.out.println("Downloaded " + (int) (downloaded / 1024 + 0.5) + " kbytes");
    }

    public String pspace(String in)
            throws IOException {
        if (in != null) {
            int b,e;
            in = StringUtils.replaceAllCase(in, "&nbsp;", " ");
            in = StringUtils.replaceAllCase(in, "&quot;", "\"");
            in = StringUtils.replaceAllCase(in, "\u00A0", " ");
            in = StringUtils.replaceAllCase(in, "&amp;", "&");
            String lo = in.toLowerCase();
            try {
                b = -1;
                while (true) { // char
                    b = lo.indexOf("&#", b + 1);
                    if (b == -1) break;
                    e = lo.indexOf(";", b);
                    if (e == -1) break;
                    String ts = in.substring(b + 2, e);
                    int t = Integer.parseInt(ts);
                    byte[] bt = new byte[1];
                    try {
                        if (t >= 128) t = t % 256 - 256;
                        bt[0] = (byte) t;
                        if (charset == null)
                            in = in.substring(0, b) + new String(bt) + in.substring(e + 1);
                        else
                            in = in.substring(0, b) + new String(bt, charset) + in.substring(e + 1);
                        lo = in.toLowerCase();
                    } catch (NumberFormatException ex) {
                    }
                }
            } catch (UnsupportedEncodingException ex) {
                throw new IOException("Unsupported Encoding : " + ex.getMessage());
            }
        }
        return in;
    }

    public boolean eof() {
        return isEOF;
    }

    public int getCurrentLineNum() {
        return CurrentLineNum;
    }

    public String getPrintBSnumber(int v, String suffix) {
        String result = "" + v + suffix;
        int l = result.length();
        for (int i = 0; i < l; i++) result += "\b";
        return result;
    }

    public void setAutoPrint(int val) {
        page = val;
    }

    public void checkPrint() {
        if (page > 0)
            if (CurrentLineNum % page == 0) System.out.print(getPrintBSnumber(CurrentLineNum, " lines"));
    }

    public String readSmallBlock(String beg, String end, String sep, String breakdata) throws java.io.IOException {
        if (in == null && isEOF) return null;
        String curr;
        int i,j,bi;

        bi = -1; // find beg in buffer, and delete bad lines
        for (i = 0; i < buffer.size(); i++) {
            curr = (String) buffer.elementAt(0);
            if (breakdata != null)
                if (curr.indexOf(breakdata) != -1) return breakdata;
            bi = curr.indexOf(beg);
            if (bi == -1)
                buffer.removeElementAt(0);
            else {
                buffer.set(0, curr.substring(bi));
                break;
            }
        }
        if (bi == -1) // find beg in file if we didn't find it in buffer
            while (!isEOF) {
                curr = pspace(addcount(readLinePatch()));
                CurrentLineNum++;
                checkPrint();
                if (curr != null) {
                    if (breakdata != null)
                        if (curr.indexOf(breakdata) != -1) return breakdata;
                    bi = curr.indexOf(beg);
                    if (bi != -1) {
                        buffer.add(curr.substring(bi));
                        break;
                    }
                } else
                    isEOF = true;
            }
        if (buffer.size() == 0 || bi == -1) return null; // we can't find beg

        String all = "";
        while (true) {
            if (buffer.size() == 0) {
                curr = pspace(addcount(readLinePatch()));
                CurrentLineNum++;
                checkPrint();
                if (curr == null) {
                    isEOF = true;
                    break;
                }
                buffer.add(curr);
            }
            curr = (String) buffer.elementAt(0);
            bi = curr.indexOf(end);
            if (bi != -1) {
                all = all + curr.substring(0, bi + end.length());
                buffer.set(0, curr.substring(bi + end.length()));
                break;
            } else {
                all = all + curr + sep;
                buffer.removeElementAt(0);
            }
        }
        while ((bi = all.indexOf(beg, 1)) != -1) // cut very small block
            all = all.substring(bi);
        return all;
    }

    public void pushString(String str) {
        buffer.add(str);
    }

    protected String readLinePatch()
            throws java.io.IOException {
        String res = oldline = in.readLine();
        if (patch.containsKey(res))
            res = (String) patch.get(res);
        return res;
    }

    public String readLine()
            throws java.io.IOException {
        if (isEOF) return null;
        String res;
        if (buffer.size() == 0) {
            res = pspace(addcount(readLinePatch()));
            isEOF = res == null;
            CurrentLineNum++;
        } else {
            res = oldline = (String) buffer.elementAt(0);
            buffer.removeElementAt(0);
            CurrentLineNum++;
        }
        checkPrint();
        return res;
    }

    public void OpenFILE(String filename)
            throws IOException {
        Close();
        page = 0;
        CurrentLineNum = 0;
        if (charset != null)
            in = new BufferedReader(new InputStreamReader(new FileInputStream(filename), charset));
        else
            in = new BufferedReader(new FileReader(filename));
        isEOF = false;
        buffer = new Vector();
    }

    public void resetCookies() {
        cookies = null;
    }
    
    protected void retrieveCookies(HttpURLConnection conn) {
        if (cookies == null)
            cookies = new TreeMap();
        String key, value;
        int i = 1;
        while ((key = conn.getHeaderFieldKey(i)) != null) {
            if (key.toLowerCase().equals("set-cookie")) {
                value = conn.getHeaderField(i);
                if (value != null) {
                    int r = value.indexOf(';');
                    if (r != -1)
                        value = value.substring(0, r);
                    r = value.indexOf('=');
                    if (r != -1)
                        cookies.put(value.substring(0, r), value.substring(r + 1));
                }
            }
            i++;
        }
    }

    protected void applyCookies() {
        if (cookies == null) return;
        Iterator it = cookies.keySet().iterator();
        String c,v,s = "",result = "";
        while (it.hasNext()) {
            c = (String) it.next();
            v = (String) cookies.get(c);
            result += s + c + "=" + v;
            s = "; ";
        }
        setHeader(HEADER_COOKIE, result);
    }

    public void setHeader(String key, String value) {
        if (headers == null)
            headers = new TreeMap();
        if (value != null)
            headers.put(key, value);
        else
            headers.remove(key);
    }

    public void setCharset(String charset) {
        this.charset = charset;
    }

    private void applyHeaders(HttpURLConnection conn) {
        if (headers == null) return;
        Iterator it = headers.keySet().iterator();
        while (it.hasNext()) {
            String key = (String) it.next();
            String value = (String) headers.get(key);
            conn.setRequestProperty(key, value);
        }
    }

    protected HttpURLConnection newConn(String URL)
            throws IOException {
        URL u = new URL(URL);
        HttpURLConnection result = (HttpURLConnection) u.openConnection();
        result.setFollowRedirects(false);
        applyCookies();
        applyHeaders(result);
        result.setUseCaches(false);
        return result;
    }

    public void openHTTP(String URL) throws IOException {
      openHTTP(URL,null, false);
    }
  public void openHTTPfake(String URL) throws IOException {
    openHTTP(URL,null, true);
  }

  public void OpenHTTP(String URI, String postData) throws IOException {
    openHTTP(URI, postData, false);
  }

    protected void openHTTP(String URI, String postData, boolean fake) throws IOException {
        Close();
        page = 0;
        CurrentLineNum = 0;

        HttpURLConnection hpCon = newConn(URI);

        if (postData != null) {
            hpCon.setDoOutput(true);
            hpCon.setRequestMethod("POST");
            hpCon.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            hpCon.setRequestProperty("Content-Length", String.valueOf(postData.length()));
            OutputStream os = hpCon.getOutputStream();
            os.write(postData.getBytes());
            os.flush();
            os.close();
            os = null;
        }

        // redirect with cookies support
        hpCon.getResponseCode();
        retrieveCookies(hpCon);
        if (hpCon.getResponseCode() == HttpURLConnection.HTTP_MOVED_TEMP) {
            String newloc = hpCon.getHeaderField("Location");
            if (newloc != null) {
                if (newloc.startsWith("http://")) { // other URL
                    URI = newloc;
                } else if (newloc.startsWith("/")) { // absolute path
                    int l = URI.indexOf('/', 8);
                    if (l != -1)
                        URI = URI.substring(0, l) + newloc;
                } else { // relative path
                    int l = URI.lastIndexOf("/");
                    if (l != -1)
                        URI = URI.substring(0, l) + "/" + newloc;
                }
            }
            hpCon = newConn(URI);
        }

        hpCon.getResponseCode();
        retrieveCookies(hpCon);
      if (!fake) {
        if (hpCon.getResponseCode() == HttpURLConnection.HTTP_OK) {
            InputStream input = hpCon.getInputStream();
            if (charset != null)
                in = new BufferedReader(new InputStreamReader(input, charset));
            else
                in = new BufferedReader(new InputStreamReader(input));
            isEOF = false;
        } else {
            if (hpCon.getResponseCode() == HttpURLConnection.HTTP_NOT_FOUND) {
                in = null;
                isEOF = true;
            } else
                throw new IOException("Error connect ! Code=" + hpCon.getResponseCode() + "/" + hpCon.getResponseMessage());
        }
        buffer = new Vector();
      } else {
        in = null;
        isEOF = true;
        buffer = null;
      }
    }

    public void Close() {
        in = null;
        buffer = null;
    }

    public void setProxy(String Host, int Port, String Auth) {
        System.getProperties().put("proxySet", "true");
        System.getProperties().put("proxyHost", Host);
        System.getProperties().put("proxyPort", Integer.toString(Port));
        if (Auth != null) {
            sun.misc.BASE64Encoder enc = new sun.misc.BASE64Encoder();
            setHeader(HEADER_PROXY_AUTH, "Basic " + enc.encode(Auth.getBytes()));
        } else {
            setHeader(HEADER_PROXY_AUTH, null);
        }
    }

    protected byte[] convert(String data, String charsetName)
            throws IOException {
        byte[] hex = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
        try {
            byte[] o = data.getBytes(charsetName);
            byte[] res = new byte[o.length * 3];
            int j = 0,v;
            for (int i = 0; i < o.length; i++) {
                if (o[i] == ' ') {
                    res[j++] = '+';
                } else if (o[i] < 0 || o[i] == ',' || o[i] == ':' || o[i] == '!' || o[i] == '+' || o[i] == '\r' || o[i] == '\n') {
                    if (o[i] < 0) v = 256 + o[i]; else v = o[i];
                    res[j++] = '%';
                    res[j++] = hex[v / 16];
                    res[j++] = hex[v % 16];
                } else {
                    res[j++] = o[i];
                }
            }
            byte[] out = new byte[j];
            for (int i = 0; i < j; i++) out[i] = res[i];
            return out;
        } catch (UnsupportedEncodingException e) {
            throw new IOException("Unsupported Encoding : " + e.getMessage());
        }
    }

  public String getCookie(String key) {
    if (cookies == null) return null;
    return (String)cookies.get(key);
  }
}
