/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.4 $
 *     $Date: 2004/03/31 10:44:12 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.text.*;
import java.util.zip.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvFinternal extends tvSuperF {
    static protected Pattern reSiteBeg,reChannelBeg,reProgramBeg,reProgramName,reProgramAnn,reName;
    static protected HashMap charconv = null;
    static protected String format = "xml";

    public void setProperties(AProperties p) {
        try {
            reSiteBeg = Pattern.compile("<site name=\"(.*)\">");
            reChannelBeg = Pattern.compile("<channel name=\"(.*)\">");
            reProgramBeg = Pattern.compile("<program time=\"(\\d{4})-(\\d{1,2})-(\\d{1,2}) (\\d{1,2}):(\\d{1,2}) GMT\" length=\"(.*)\">");
            reProgramName = Pattern.compile("<name>(.*)</name>");
            reProgramAnn = Pattern.compile("<announcement>(.*)</announcement>");

            format = p.getProperty("internal.format", "xml");
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE in tvFinternal : " + e.getMessage());
            System.exit(1);
        }
    }

    protected String toXML(String in) {
        in = StringUtils.replaceAllCase(in, "&", "&amp;");
        in = StringUtils.replaceAllCase(in, "<", "&lt;");
        in = StringUtils.replaceAllCase(in, ">", "&gt;");
        in = StringUtils.replaceAllCase(in, "\n", "&#13;");
      in = StringUtils.replaceAllCase(in, "\"", "&quot;");
      in = StringUtils.replaceAllCase(in, "'", "&apos;");

        return in;
    }

    protected String fromXML(String in) {
        in = StringUtils.replaceAllCase(in, "&#13;", "\n");
        in = StringUtils.replaceAllCase(in, "&lt;", "<");
        in = StringUtils.replaceAllCase(in, "&gt;", ">");
      in = StringUtils.replaceAllCase(in, "&quot;", "\"");
      in = StringUtils.replaceAllCase(in, "&apos;", "'");
        in = StringUtils.replaceAllCase(in, "&amp;", "&");
        return in;
    }

    static protected void findfiles(String dir, ArrayList fnames, String begName, String endName) {
        File all[] = new File(dir).listFiles();
        if (all != null)
            for (int i = 0; i < all.length; i++)
                if (!all[i].isDirectory()) {
                    String filename = all[i].getPath();
                    if (filename.startsWith(begName) && filename.endsWith(endName))
                        fnames.add(filename);
                } else
                    findfiles(all[i].getPath(), fnames, begName, endName);
    }

    static public void mkdirs(String filename) {
        int z = filename.lastIndexOf(File.separator);
        if (z == -1) return;
        new File(filename.substring(0, z)).mkdirs();
    }

    static public String getFNbyCN(String ChannelName) {
        if (charconv == null) {
            charconv = new HashMap();
            charconv.put(new Character('/'), "-");
            charconv.put(new Character('"'), "");
            charconv.put(new Character(' '), "_");
            charconv.put(new Character('('), "-");
            charconv.put(new Character(')'), "");
            charconv.put(new Character('-'), "-");
            charconv.put(new Character('+'), "-");
            charconv.put(new Character('&'), "n");
            charconv.put(new Character('!'), "");
            charconv.put(new Character('.'), ".");
            charconv.put(new Character(','), ",");

            charconv.put(new Character('q'), "q");
            charconv.put(new Character('w'), "w");
            charconv.put(new Character('e'), "e");
            charconv.put(new Character('r'), "r");
            charconv.put(new Character('t'), "t");
            charconv.put(new Character('y'), "y");
            charconv.put(new Character('u'), "u");
            charconv.put(new Character('i'), "i");
            charconv.put(new Character('o'), "o");
            charconv.put(new Character('p'), "p");
            charconv.put(new Character('a'), "a");
            charconv.put(new Character('s'), "s");
            charconv.put(new Character('d'), "d");
            charconv.put(new Character('f'), "f");
            charconv.put(new Character('g'), "g");
            charconv.put(new Character('h'), "h");
            charconv.put(new Character('j'), "j");
            charconv.put(new Character('k'), "k");
            charconv.put(new Character('l'), "l");
            charconv.put(new Character('z'), "z");
            charconv.put(new Character('x'), "x");
            charconv.put(new Character('c'), "c");
            charconv.put(new Character('v'), "v");
            charconv.put(new Character('b'), "b");
            charconv.put(new Character('n'), "n");
            charconv.put(new Character('m'), "m");

            charconv.put(new Character('Q'), "Q");
            charconv.put(new Character('W'), "W");
            charconv.put(new Character('E'), "E");
            charconv.put(new Character('R'), "R");
            charconv.put(new Character('T'), "T");
            charconv.put(new Character('Y'), "Y");
            charconv.put(new Character('U'), "U");
            charconv.put(new Character('I'), "I");
            charconv.put(new Character('O'), "O");
            charconv.put(new Character('P'), "P");
            charconv.put(new Character('A'), "A");
            charconv.put(new Character('S'), "S");
            charconv.put(new Character('D'), "D");
            charconv.put(new Character('F'), "F");
            charconv.put(new Character('G'), "G");
            charconv.put(new Character('H'), "H");
            charconv.put(new Character('J'), "J");
            charconv.put(new Character('K'), "K");
            charconv.put(new Character('L'), "L");
            charconv.put(new Character('Z'), "Z");
            charconv.put(new Character('X'), "X");
            charconv.put(new Character('C'), "C");
            charconv.put(new Character('V'), "V");
            charconv.put(new Character('B'), "B");
            charconv.put(new Character('N'), "N");
            charconv.put(new Character('M'), "M");

            charconv.put(new Character('й'), "y");
            charconv.put(new Character('ц'), "c");
            charconv.put(new Character('у'), "u");
            charconv.put(new Character('к'), "k");
            charconv.put(new Character('е'), "e");
            charconv.put(new Character('н'), "n");
            charconv.put(new Character('г'), "g");
            charconv.put(new Character('ш'), "sh");
            charconv.put(new Character('щ'), "sch");
            charconv.put(new Character('з'), "z");
            charconv.put(new Character('х'), "h");
            charconv.put(new Character('ъ'), "");
            charconv.put(new Character('ф'), "f");
            charconv.put(new Character('ы'), "i");
            charconv.put(new Character('в'), "v");
            charconv.put(new Character('а'), "a");
            charconv.put(new Character('п'), "p");
            charconv.put(new Character('р'), "r");
            charconv.put(new Character('о'), "o");
            charconv.put(new Character('л'), "l");
            charconv.put(new Character('д'), "d");
            charconv.put(new Character('ж'), "zh");
            charconv.put(new Character('э'), "e");
            charconv.put(new Character('я'), "ya");
            charconv.put(new Character('ч'), "ch");
            charconv.put(new Character('с'), "s");
            charconv.put(new Character('м'), "m");
            charconv.put(new Character('и'), "i");
            charconv.put(new Character('т'), "t");
            charconv.put(new Character('ь'), "");
            charconv.put(new Character('б'), "b");
            charconv.put(new Character('ю'), "yu");
            charconv.put(new Character('ё'), "yo");

            charconv.put(new Character('Й'), "Y");
            charconv.put(new Character('Ц'), "C");
            charconv.put(new Character('У'), "U");
            charconv.put(new Character('К'), "K");
            charconv.put(new Character('Е'), "E");
            charconv.put(new Character('Н'), "N");
            charconv.put(new Character('Г'), "G");
            charconv.put(new Character('Ш'), "SH");
            charconv.put(new Character('Щ'), "SCH");
            charconv.put(new Character('З'), "Z");
            charconv.put(new Character('Х'), "H");
            charconv.put(new Character('Ъ'), "");
            charconv.put(new Character('Ф'), "F");
            charconv.put(new Character('Ы'), "I");
            charconv.put(new Character('В'), "V");
            charconv.put(new Character('А'), "A");
            charconv.put(new Character('П'), "P");
            charconv.put(new Character('Р'), "R");
            charconv.put(new Character('О'), "O");
            charconv.put(new Character('Л'), "L");
            charconv.put(new Character('Д'), "D");
            charconv.put(new Character('Ж'), "ZH");
            charconv.put(new Character('Э'), "E");
            charconv.put(new Character('Я'), "YA");
            charconv.put(new Character('Ч'), "CH");
            charconv.put(new Character('С'), "S");
            charconv.put(new Character('М'), "M");
            charconv.put(new Character('И'), "I");
            charconv.put(new Character('Т'), "T");
            charconv.put(new Character('Ь'), "");
            charconv.put(new Character('Б'), "B");
            charconv.put(new Character('Ю'), "YU");
            charconv.put(new Character('Ё'), "YA");

            charconv.put(new Character('è'), "e");
            charconv.put(new Character('ê'), "e");
            charconv.put(new Character('é'), "e");
            charconv.put(new Character('è'), "e");
            charconv.put(new Character('è'), "e");
            charconv.put(new Character('è'), "e");
            charconv.put(new Character('è'), "e");

            charconv.put(new Character('0'), "0");
            charconv.put(new Character('1'), "1");
            charconv.put(new Character('2'), "2");
            charconv.put(new Character('3'), "3");
            charconv.put(new Character('4'), "4");
            charconv.put(new Character('5'), "5");
            charconv.put(new Character('6'), "6");
            charconv.put(new Character('7'), "7");
            charconv.put(new Character('8'), "8");
            charconv.put(new Character('9'), "9");

            charconv.put(new Character('*'), "-");
        }
        char c;
        String cz;
        String result = "";
        for (int i = 0; i < ChannelName.length(); i++) {
            c = ChannelName.charAt(i);
            cz = (String) charconv.get(new Character(c));
            if (cz == null) {
                System.err.println("Warning: Error convert char to filename : '" + c + "' in string \"" + ChannelName + "\"");
            } else
                result += cz;
        }
        return result;
    }

    public void saveToFile(String filename, AllChannels list)
            throws FileNotFoundException, IOException {
        String outformat = format;
        if (filename.equals("--format=xml")) outformat = "xml";
        if (filename.equals("--format=gzxml")) outformat = "gzxml";
        if (filename.equals("--format=ser")) outformat = "ser";
        if (filename.equals("--format=gzser")) outformat = "gzser";
        if (filename.startsWith("--format="))
            filename = TVProgram.getNextArg();


        if (outformat.equals("xml") || outformat.equals("gzxml"))
            saveToFileXML(filename, list, format.equals("gzxml"));
        if (outformat.equals("ser") || outformat.equals("gzser"))
            saveToFileSer(filename, list, format.equals("gzser"));
    }

    public void loadFromFile(String filename, AllChannels list)
            throws FileNotFoundException, IOException {
        String outformat = format;
        if (filename.equals("--format=xml")) outformat = "xml";
        if (filename.equals("--format=gzxml")) outformat = "gzxml";
        if (filename.equals("--format=ser")) outformat = "ser";
        if (filename.equals("--format=gzser")) outformat = "gzser";
        if (filename.startsWith("--format="))
            filename = TVProgram.getNextArg();

        if (outformat.equals("xml") || outformat.equals("gzxml"))
            loadFromFileXML(filename, list, format.equals("gzxml"));
        if (outformat.equals("ser") || outformat.equals("gzser"))
            loadFromFileSer(filename, list, format.equals("gzser"));
    }

    public void saveToFileSer(String filename, AllChannels list, boolean packed)
            throws FileNotFoundException, IOException {
        ObjectOutputStream wr;
        OneSite site;
        for (int i = 0; i < list.size(); i++) {
            site = list.elementAt(i);
            mkdirs(filename + site.SiteName);
            if (packed)
                wr = new ObjectOutputStream(new GZIPOutputStream(new FileOutputStream(filename + File.separator + site.SiteName + ".ser.gz", false), 900 * 1024));
            else
                wr = new ObjectOutputStream(new FileOutputStream(filename + File.separator + site.SiteName + ".ser", false));
            try {
                wr.writeObject(site);
            } catch (Exception e) {
                System.err.println("Exception " + e.getClass().getName() + "/" + e.getMessage());
                System.exit(2);
            }
            wr.close();
        }
    }

    public void saveToFileXML(String filename, AllChannels list, boolean packed)
            throws FileNotFoundException, IOException {
        NumberFormat n2 = NumberFormat.getInstance();
        n2.setMinimumIntegerDigits(2);
        n2.setGroupingUsed(false);
        NumberFormat n4 = NumberFormat.getInstance();
        n4.setMinimumIntegerDigits(4);
        n4.setGroupingUsed(false);
        OneChannel ch;
        Prog pr;
        for (int k = 0; k < list.size(); k++) {
            for (int i = 0; i < list.elementAt(k).size(); i++) {
                ch = list.elementAt(k).elementAt(i);

                String fn;

                fn = filename + list.elementAt(k).SiteName + File.separator + getFNbyCN(ch.ChannelName) + ".xml";
                if (packed)
                    fn += ".gz";
                new File(fn).delete();
                mkdirs(fn);
                if (ch.size() > 0) {
                    BufferedWriter wr;
                    if (packed)
                        wr = new BufferedWriter(new OutputStreamWriter(new GZIPOutputStream(new FileOutputStream(fn, false), 900 * 1024), "UTF-8"));
                    else
                        wr = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(fn, false), "UTF-8"));
                    wr.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
                    wr.write("<!--Data for ATV program(http://zaval.org/products/atv)-->\n");
                    wr.write("<!DOCTYPE site SYSTEM \"http://zaval.org/products/atv/atv.dtd\">\n");
                    wr.write("<site name=\"" + toXML(list.elementAt(k).SiteName) + "\">\n");
                    wr.write(" <channel name=\"" + toXML(ch.ChannelName) + "\">\n");
                    for (int j = 0; j < ch.size(); j++) {
                        pr = ch.elementAt(j);
                        int dom,month,year,h,m;
                        Calendar c = Calendar.getInstance();
                        c.setTimeZone(TimeZone.getTimeZone("GMT"));
                        c.setTime(new Date(pr.BegTime));
                        dom = c.get(Calendar.DAY_OF_MONTH);
                        month = c.get(Calendar.MONTH) + 1;
                        year = c.get(Calendar.YEAR);
                        h = c.get(Calendar.HOUR_OF_DAY);
                        m = c.get(Calendar.MINUTE);

                        wr.write("  <program time=\"" + n4.format(year) + "-" + n2.format(month) + "-" + n2.format(dom) + " " + n2.format(h) + ":" + n2.format(m) + " GMT\" length=\"" + pr.LengthTime + "\">\n");
                        wr.write("   <name>" + toXML(pr.ProgramName) + "</name>\n");
                        if (pr.Announcement != null) {
                            wr.write("   <announcement>" + toXML(pr.Announcement) + "</announcement>\n");
                        }
                        wr.write("  </program>\n");
                    }
                    wr.write(" </channel>\n");
                    wr.write("</site>\n");
                    wr.flush();
                    wr.close();
                }
            }
        }
    }

    public void loadFromFileSer(String filename, AllChannels list, boolean packed)
            throws FileNotFoundException, IOException {
        ArrayList flist = new ArrayList();
        String endName;
        if (packed)
            endName = ".ser.gz";
        else
            endName = ".ser";
        findfiles(filename, flist, filename, endName);

        ObjectInputStream rd;

        for (int i = 0; i < flist.size(); i++) {
            if (packed)
                rd = new ObjectInputStream(new GZIPInputStream(new FileInputStream((String) flist.get(i))));
            else
                rd = new ObjectInputStream(new FileInputStream((String) flist.get(i)));
            try {
                list.add((OneSite) rd.readObject());
            } catch (ClassNotFoundException e) {
                System.err.println("Error read data file : " + e.getMessage());
                System.exit(2);
            }
            rd.close();
        }
    }

    public void loadFromFileXML(String filename, AllChannels list, boolean packed)
            throws FileNotFoundException, IOException {
        ArrayList names = new ArrayList();
        String endName;
        if (packed)
            endName = ".xml.gz";
        else
            endName = ".xml";
        findfiles(filename, names, filename, endName);

        String str,onename;
        OneChannel chan = null;
        OneSite site = null;
        Matcher m;
        for (int i = 0; i < names.size(); i++) {
            onename = (String) names.get(i);
            BufferedReader rd;
            if (packed)
                rd = new BufferedReader(new InputStreamReader(new GZIPInputStream(new FileInputStream(onename)), "UTF-8"));
            else
                rd = new BufferedReader(new InputStreamReader(new FileInputStream(onename), "UTF-8"));
            long progbegtime = 0;
            int proglen = 0;
            String progname = null,progann = null;
            while ((str = rd.readLine()) != null) {
                str = str.trim();
                if (str.equals("</program>")) {
                    if (progbegtime <= 0 || proglen < 0 || progname == null)
                        throw new IOException("Error format atv data file !!!");
                    chan.add(new Prog(progbegtime, proglen, progname, progann));
                    progname = null;
                    progbegtime = 0;
                    proglen = 0;
                    progann = null;
                } else if (str.equals("</site>")) {
                    site = null;
                    chan = null;
                    progname = null;
                    progbegtime = 0;
                    proglen = 0;
                    progann = null;
                } else if (str.equals("</channel>")) {
                    chan = null;
                    progname = null;
                    progbegtime = 0;
                    proglen = 0;
                    progann = null;
                } else if ((m = reSiteBeg.matcher(str)).find()) {
                    site = list.findcreate(fromXML(m.group(1)));
                    chan = null;
                    progname = null;
                    progbegtime = 0;
                    proglen = 0;
                    progann = null;
                } else if ((m = reChannelBeg.matcher(str)).find()) {
                    chan = new OneChannel(fromXML(m.group(1)));
                    site.add(chan);
                    progname = null;
                    progbegtime = 0;
                    proglen = 0;
                    progann = null;
                } else if ((m = reProgramBeg.matcher(str)).find()) {
                    int dom,month,year,h,ms;
                    year = Integer.parseInt(m.group(1));
                    month = Integer.parseInt(m.group(2));
                    dom = Integer.parseInt(m.group(3));
                    h = Integer.parseInt(m.group(4));
                    ms = Integer.parseInt(m.group(5));

                    Calendar c = Calendar.getInstance();
                    c.setTimeZone(TimeZone.getTimeZone("GMT"));
                    c.set(Calendar.YEAR, year);
                    c.set(Calendar.MONTH, month - 1);
                    c.set(Calendar.DAY_OF_MONTH, dom);
                    c.set(Calendar.HOUR_OF_DAY, h);
                    c.set(Calendar.MINUTE, ms);
                    c.set(Calendar.SECOND, 0);
                    c.set(Calendar.MILLISECOND, 0);
                    progbegtime = c.getTime().getTime();
                    proglen = Integer.parseInt(m.group(6));
                    progname = null;
                    progann = null;
                } else if ((m = reProgramName.matcher(str)).find()) {
                    progname = fromXML(m.group(1));
                } else if ((m = reProgramAnn.matcher(str)).find()) {
                    progann = fromXML(m.group(1));
                }
            }
        }
    }
}



