/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:58 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.io.*;
import java.util.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvFjtv extends tvSuperF {
    protected AOutputStream wrndx,wrpdt;
    protected AInputStream rdndx;
    byte rdpdt[];
    protected int DateDelta = 134774;
    protected String charset = null;
    protected short deflen;
    public static final byte[] sig = new String("JTV 3.x TV Program Data\n\n\n").getBytes();


    public void setProperties(AProperties p) {
        charset = p.getProperty("jtv.charset");
        deflen = (short) Integer.parseInt(p.getProperty("list.defaultlength", "60"));
    }

    public void saveToFile(String filename, AllChannels list)
            throws FileNotFoundException, IOException {
        OneChannel cp;
        OneSite s;
        if (filename == null) filename = "";
        for (int i = 0; i < list.size(); i++) {
            s = list.elementAt(i);
            for (int j = 0; j < s.size(); j++) {
                cp = s.elementAt(j);
                wrndx = new AOutputStream(new BufferedOutputStream(new FileOutputStream(filename + cp.ChannelName + ".ndx")), false);
                wrpdt = new AOutputStream(new BufferedOutputStream(new FileOutputStream(filename + cp.ChannelName + ".pdt")), false);
                saveChannelProgTo(cp);
                wrndx.close();
                wrpdt.close();
            }
        }
    }

    public void realLoad(String filename, AllChannels list)
            throws FileNotFoundException, IOException, EOFException {
        Vector jlist = new Vector();
        long d;
        short offset;
        String n;
        Prog lastprog = null;
        long lasttime = 0;

        rdndx = new AInputStream(new FileInputStream(filename + ".ndx"), false);

        AInputStream frdpdt = new AInputStream(new FileInputStream(filename + ".pdt"), false);
        rdpdt = new byte[(int) (new File(filename + ".pdt").length())];
        frdpdt.read(rdpdt);
        frdpdt.close();

        if (!readSig()) throw new IOException("Bad file format");
        short num = rdndx.readShort();
        int p = filename.lastIndexOf(File.separator);
        String cname;
        if (p == -1)
            cname = filename;
        else
            cname = filename.substring(p + 1);
        OneChannel cp = new OneChannel(cname);
        list.findcreate("jtv").add(cp);
        for (int i = 0; i < num; i++) {
            if (rdndx.readShort() != 0) throw new IOException("Bad file format");
            d = readDate();
            d -= TimeEngine.getOffset(d, TimeEngine.localTZ);
            offset = rdndx.readShort();
            n = readSPasString(offset);
            if (lastprog != null)
                lastprog.LengthTime = (short) ((d - lasttime) / 1000L / 60);
            lastprog = new Prog(d, 0, n, null);
            cp.add(lastprog);
            lasttime = d;
        }
        if (lastprog != null)
            lastprog.LengthTime = deflen;
        rdndx.close();
    }

    class dirFilter implements FilenameFilter {
        public boolean accept(File dir, String name) {
            return name.endsWith(".ndx");
        }
    }

    public void loadFromFile(String filename, AllChannels list)
            throws FileNotFoundException, IOException, EOFException {
        if (!filename.startsWith("--fromdir="))
            realLoad(filename, list);
        else {
            String dir = filename.substring(10);
            String names[] = new File(dir).list(new dirFilter());
            String onename;
            for (int i = 0; i < names.length; i++) {
                onename = names[i].substring(0, names[i].length() - 4);
                System.out.print("Loading " + onename + ".{pdt,ndx} ...");
                realLoad(dir + File.separator + onename, list);
                System.out.println("done");
            }
        }
    }

    protected void writeDate(long dt)
            throws IOException {
        wrndx.writeLong((dt + DateDelta * 24L * 60 * 60 * 1000) * 10000);
    }

    protected long readDate()
            throws IOException, EOFException {
        long v = rdndx.readLong();

        return v / 10000 - DateDelta * 24L * 60 * 60 * 1000;
    }

    protected void saveChannelProgTo(OneChannel cp)
            throws IOException {
        short offset = (short) sig.length;
        Prog p;

        wrpdt.write(sig);
        wrndx.writeShort((short) cp.size());
        for (int i = 0; i < cp.size(); i++) {
            p = cp.elementAt(i);
            wrndx.writeShort((short) 0);
            long d = p.BegTime;
            d += TimeEngine.getOffset(d, TimeEngine.localTZ);
            writeDate(p.BegTime);
            wrndx.writeShort(offset);
            offset += writeSPasString(p.ProgramName);
        }
    }

    protected int writeSPasString(String str)
            throws IOException {
        if (str == null) {
            wrpdt.writeShort((short) 0);
            return 2;
        } else {
            wrpdt.writeShort((short) str.length());
            byte sb[];
            if (charset == null)
                sb = str.getBytes();
            else
                sb = str.getBytes(charset);
            wrpdt.write(sb);
            return 2 + sb.length;
        }
    }

    protected boolean readSig()
            throws IOException, EOFException {
        if (sig.length > rdpdt.length) throw new IOException("Bad file format");
        for (int i = 0; i < sig.length; i++)
            if (sig[i] != rdpdt[i]) return false;
        return true;
    }

    protected String readSPasString(short offset)
            throws IOException {
        int b1,b0;
        if ((offset + 1) > rdpdt.length) throw new IOException("Bad file format1");
        b0 = rdpdt[offset];
        b1 = rdpdt[offset + 1];
        if (b0 < 0) b0 = 256 + b0;
        if (b1 < 0) b1 = 256 + b1;
        int len = b0 + (b1 << 8);
        offset += 2;
        if ((offset + len) > rdpdt.length) throw new IOException("Bad file format2");
        if (len <= 0) throw new IOException("Bad file format3");
        byte o[] = new byte[len];
        for (int i = 0; i < len; i++, offset++)
            o[i] = rdpdt[offset];

        if (charset == null)
            return new String(o);
        else
            return new String(o, charset);
    }
}
