/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:02 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.io.*;
import java.util.*;

import org.zaval.atv.*;
import org.zaval.util.io.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvFlisttv extends tvSuperF {
    protected AOutputStream wr;
    protected AInputStream rd;
    public static final int DateDelta = 25569;
    String charset = null;

    public void setProperties(AProperties p) {
        charset = p.getProperty("listtv.charset");
    }

    public void saveToFile(String filename, AllChannels list)
            throws FileNotFoundException, IOException {
        wr = new AOutputStream(new BufferedOutputStream(new FileOutputStream(filename, false)), false);

        for (int i = 0; i < list.size(); i++)
            for (int j = 0; j < list.elementAt(i).size(); j++)
                saveChannelProgTo(list.elementAt(i).elementAt(j));
        wr.flush();
        wr.close();
    }

    public void loadFromFile(String filename, AllChannels list)
            throws FileNotFoundException, IOException {
        rd = new AInputStream(new BufferedInputStream(new FileInputStream(filename)), false);
        try {
            while (true) loadChannelProgFrom(list);
        } catch (EOFException e) {
            rd.close();
        }
    }

    protected long ddTOjd(int dt) {
        return (dt - DateDelta) * 24L * 60 * 60 * 1000;
    }

    protected void saveChannelProgTo(OneChannel cp)
            throws IOException {
        Prog pr;
        // make days list
        HashMap daylist = new HashMap();
        int day;
        Integer dcount;
        for (int i = 0; i < cp.size(); i++) { // make day list
            pr = cp.elementAt(i);

            day = (int) ((pr.BegTime + TimeEngine.getOffset(pr.BegTime, TimeEngine.localTZ)) / 1000 / 60 / 60 / 24);
            dcount = (Integer) daylist.get(new Integer(day));
            if (dcount == null)
                dcount = new Integer(1);
            else
                dcount = new Integer(dcount.intValue() + 1);
            daylist.put(new Integer(day), dcount);
        }
        Object[] sdaylist = daylist.keySet().toArray();
        Arrays.sort(sdaylist);

        for (int i = 0; i < sdaylist.length; i++) {
            wr.writeInt(((Integer) sdaylist[i]).intValue() + DateDelta);
            wr.writePasString(cp.ChannelName, charset);
            dcount = (Integer) daylist.get(sdaylist[i]);
            wr.writeInt(dcount.intValue());
            for (int j = 0; j < cp.size(); j++) {
                pr = cp.elementAt(j);
                day = (int) ((pr.BegTime + TimeEngine.getOffset(pr.BegTime, TimeEngine.localTZ)) / 1000 / 60 / 60 / 24);
                if (day != ((Integer) sdaylist[i]).intValue()) continue;
                wr.writeInt(0);  // рейтинг
                wr.writeByte(0); // признак ручной установки рейтинга
                wr.writeInt(0);  // тип передачи
                wr.writeByte(0); // признак ручной установки типа
                wr.writeInt((int) (((pr.BegTime + TimeEngine.getOffset(pr.BegTime, TimeEngine.localTZ)) % (1000L * 60 * 60 * 24)) / 1000 / 60));// время начала в минутах
                wr.writeInt(pr.LengthTime); // длительность в минутах
                wr.writePasString(pr.ProgramName, charset); // текст передачи
                wr.writePasString(pr.Announcement, charset); // текст анонса
            }
        }
    }

    protected void loadChannelProgFrom(AllChannels list)
            throws EOFException, IOException {
        int i,sz;
        short len;
        String str,ann;
        long tm;

        long dt = ddTOjd(rd.readInt());
        dt -= TimeEngine.getOffset(dt, TimeEngine.localTZ);
        str = rd.readPasString(charset);
        OneChannel cp = new OneChannel(str);
        list.findcreate("listtv").add(cp);
        sz = rd.readInt();
        for (i = 0; i < sz; i++) {
            rd.readInt();    // рейтинг
            rd.readByte();   // признак ручной установки рейтинга
            rd.readInt();    // тип передачи
            rd.readByte(); // признак ручной установки типа
            tm = rd.readInt() * 60L * 1000;// время начала в минутах
            len = (short) rd.readInt(); // длительность в минутах
            str = rd.readPasString(charset); // текст передачи
            ann = rd.readPasString(charset);
            cp.add(new Prog(dt + tm, len, str, ann));
        }
    }
}
