/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:59 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import org.zaval.atv.*;
import org.zaval.util.io.*;

import java.io.*;
import java.util.*;
import java.util.regex.*;

import org.gjt.jjt.pdbfile.*;

public class tvFpalm extends tvSuperF {
    AOutputStream wr = null;
    AInputStream rd = null;
    public static final long TimeDelta = 24107L * 24 * 60 * 60;
    public static final int maxRecSize = 64000;
    public static final String hPDBName = "all";
    public static final short hVersion = 1;
    public static final byte[] hCreatorID = {(byte) 'a', (byte) 'b', (byte) 'T', (byte) 'V'};
    public static final byte[] hDatabaseType = {(byte) 'D', (byte) 'a', (byte) 't', (byte) 'a'};
    String defaultcharset = null,charset;
    protected Pattern reName;
    protected AProperties props = null;

    public void setProperties(AProperties p) {
        defaultcharset = p.getProperty("palm.charset");
        props = p;
        try {
            reName = Pattern.compile("^.+\\((.+)\\)$");
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE in tvFpalm : " + e.getMessage());
            System.exit(1);
        }
    }

    protected void setCharset(String site) {
        charset = props.getProperty("palm.charset." + site);
        if (charset == null) charset = defaultcharset;
    }

    protected String test32(String name) {
        if (name.length() > 32) {
            return name.substring(0, 27) + (name.hashCode() % 10000);
        } else
            return name;
    }

    public void loadFromFile(String filename, AllChannels list)
            throws IOException, PTimeException, ProgException {
        if (filename == null) filename = "TVprogram.pdb";
        charset = defaultcharset;
        PDBFile pdb = new PDBFile(filename, charset);
        try {
            pdb.readFile(new RawAddable());
        } catch (IllegalAccessException e) {
            System.err.println("Module palm : Illegal Access Exception : " + e.getMessage());
            System.exit(3);
        } catch (InstantiationException e) {
            System.err.println("Module palm : Instatiation Exception : " + e.getMessage());
            System.exit(3);
        }
        if (pdb.getVersion() != hVersion ||
                !(new String(pdb.getCreatorID()).equals(new String(hCreatorID))) ||
                !(new String(pdb.getDatabaseType()).equals(new String(hDatabaseType))))
            throw new ProgException("Bad file format");
        Vector recs = pdb.getRecords();
        for (int i = 0; i < recs.size(); i++) {
            Addable a = (Addable) recs.elementAt(i);
            ByteArrayInputStream ba = new ByteArrayInputStream(a.getBytes());
            rd = new AInputStream(ba, true);
            loadChannelProgFrom(list);
        }
    }

    protected void saveChannels(ArrayList channels, ArrayList sitenames, String filename, String pdbname)
            throws IOException {
        PDBFile pdb;
        ByteArrayOutputStream ba = new ByteArrayOutputStream();
        wr = new AOutputStream(ba, true);

        new File(filename).delete();
        tvFinternal.mkdirs(filename);
        pdb = new PDBFile(filename, charset);
        String[] head = {"PDBName=" + test32("atv-" + pdbname), "Version=" + hVersion, "CreatorID=" + new String(hCreatorID), "DatabaseType=" + new String(hDatabaseType)};
        pdb.setHeaderData(head);

        for (int i = 0; i < channels.size(); i++) {
            OneChannel ch = (OneChannel) channels.get(i);
            String sitename = (String) sitenames.get(i);
            int mprev = 0;
            while (mprev < ch.size()) {
                int m = checkMaxRecSize(ch, mprev);
                ba.reset();
                saveChannelProgTo(ch, sitename, mprev, m);
                wr.flush();
                pdb.addRecord(ba.toByteArray());
                mprev = m;
            }
        }
        pdb.writeFile();

        wr = null;
    }

    public void saveToFile(String filename, AllChannels list)
            throws IOException {
        ArrayList tosave = new ArrayList();
        ArrayList tosaves = new ArrayList();
        if (filename == null) filename = "TVprogram.pdb";

        OneSite s;
        if (filename.equals("--separate-channels")) {
            filename = TVProgram.getNextArg();
            if (filename == null) filename = "";
            for (int i = 0; i < list.size(); i++) {
                s = list.elementAt(i);
                setCharset(s.SiteName);
                for (int j = 0; j < s.size(); j++) {
                    tosave.clear();
                    tosave.add(s.elementAt(j));
                    tosaves.clear();
                    tosaves.add(s.SiteName);
                    saveChannels(tosave, tosaves, filename + s.SiteName + File.separator + tvFinternal.getFNbyCN(s.elementAt(j).ChannelName) + ".pdb", tvFinternal.getFNbyCN(s.SiteName + "-" + s.elementAt(j).ChannelName));
                }
            }
        } else if (filename.equals("--separate-sites")) {
            filename = TVProgram.getNextArg();
            if (filename == null) filename = "";
            OneChannel ch;
            Matcher m;
            for (int i = 0; i < list.size(); i++) {
                s = list.elementAt(i);
                setCharset(s.SiteName);
                tosave.clear();
                tosaves.clear();
                for (int j = 0; j < s.size(); j++) {
                    tosave.add(s.elementAt(j));
                    tosaves.add(s.SiteName);
                }
                saveChannels(tosave, tosaves, filename + s.SiteName + ".pdb", s.SiteName);
            }
        } else {
            charset = defaultcharset;
            for (int i = 0; i < list.size(); i++) {
                s = list.elementAt(i);
                for (int j = 0; j < s.size(); j++) {
                    tosave.add(s.elementAt(j));
                    tosaves.add(s.SiteName);
                }
            }
            saveChannels(tosave, tosaves, filename, hPDBName);
        }
    }

    protected void loadChannelProgFrom(AllChannels list)
            throws IOException, PTimeException {
        String cn = rd.readSPasString0(charset);
        rd.read();
        if (cn.length() % 2 == 0) rd.read(); // we have \0 char after name also
        int num = rd.readInt();
        OneChannel cp = new OneChannel(cn);
        for (int i = 0; i < num; i++) {
            long begtime = readTime();
            short len = (short) (rd.readShort() / 60);
            String name = rd.readSPasString(charset);
            if (name.length() % 2 == 1) rd.read();
            String ann = rd.readSPasString(charset);
            if (ann != null)
                if (ann.length() % 2 == 1) rd.read();
            cp.add(new Prog(begtime, len, name, ann));
        }
        list.findcreate("palm").add(cp);
    }

    protected int checkMaxRecSize(OneChannel cp, int beginfrom)
            throws IOException {
        int sz = 0;
        sz += wr.calcSPasString0(cp.ChannelName, charset) + 1;
        sz += 4;
        Prog pr;
        for (int i = beginfrom; i < cp.size(); i++) {
            pr = cp.elementAt(i);
            sz += 4;
            sz += 2;
            sz += wr.calcSPasString(pr.ProgramName, charset);
            sz += wr.calcSPasString(pr.Announcement, charset);
            if (sz > maxRecSize) {
                if (i == beginfrom)
                    return i + 1;
                else
                    return i;
            }
        }
        return cp.size();
    }

    protected void saveChannelProgTo(OneChannel cp, String sitename, int from, int to)
            throws IOException {
        if (wr.writeSPasString0(cp.ChannelName + "(" + sitename + ")", charset) % 2 == 1) wr.write(0);
        wr.writeInt(to - from);
        Prog pr;
        for (int i = from; i < to; i++) {
            pr = cp.elementAt(i);
            writeTime(pr.BegTime);// время начала
            wr.writeShort(pr.LengthTime * 60); // длительность в минутах
            if (wr.writeSPasString(pr.ProgramName, charset) % 2 == 1) wr.write(0); // текст передачи
            if (wr.writeSPasString(pr.Announcement, charset) % 2 == 1) wr.write(0); // текст анонса
        }
    }

    protected long readTime()
            throws IOException {
        long tm = (rd.readUInt() - TimeDelta) * 1000;
        return tm/*-TimeEngine.getOffset(tm,TimeEngine.localTZ)*/;
    }

    protected void writeTime(long dt)
            throws IOException {
        wr.writeUInt((dt/*+TimeEngine.getOffset(dt,TimeEngine.localTZ)*/) / 1000 + TimeDelta);
    }
}
