/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:02 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.regex.*;
import java.util.*;
import java.io.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPajara extends tvSuperP {
    protected Pattern reDate,reProgramAct;
    protected TimeZone tz;
    protected long basedate = 0,oldbasedate = 0;

    tvPajara() {
        tz = null;
        try {
            reDate = Pattern.compile("\\p{L}\\s+НА\\s+(\\d{1,2})[\\.|\\-](\\d{1,2})[\\.|\\-](\\d{4})", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgramAct = Pattern.compile("<b>\\s*(\\d{1,2}[\\.|\\:]\\d{2})\\s+(.+)</b>.*<br>(.*)<br>(.*)<br>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPajara : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("ajara.TimeZone", "Asia/Tbilisi"));
        setAllowedChannels(p, "ajara");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    protected boolean parse(Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        Matcher m;
        String str;
        ParseOneChannel currcp = null;

        String daynum = null,month = null,year = null;
        long begtime;
        while ((str = tr.readLine()) != null) {
            if ((m = reDate.matcher(str)).find()) {
                int b;
                daynum = m.group(1);
                month = m.group(2);
                year = m.group(3);
                basedate = TimeEngine.parseDate(daynum, month, year, null);
                currcp = new ParseOneChannel("Ajara TV");
                list.findcreate("ajara").add(currcp);
                break;
            }
        }
        if (str == null) return false;
        while ((str = tr.readSmallBlock("<tr", "</tr>", " ", null)) != null) {
            if ((m = reProgramAct.matcher(str)).find() && currcp != null) {
                begtime = TimeEngine.parseTime(m.group(1), tz, basedate) + basedate;
                String a1 = m.group(3).trim();
                String a2 = m.group(4).trim();
                if (!a1.equals("") && !a2.equals("")) {
                    a1 = "(" + a1 + ". " + a2 + ")";
                    a2 = "";
                } else if (!a1.equals("") || !a2.equals("")) {
                    a1 = "(" + a1 + a2 + ")";
                    a2 = "";
                }
                currcp.add(new ParseProg(true, begtime, 0, m.group(2) + a1));
            }
        }
        return currcp.size() > 0;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
/*        if (!isChannelAllowed("Ajara TV")) {
            System.out.println("Loading file... skipped");
            return;
        }*/
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file... ");
        if (parse(tr, list))
            System.out.println("done         ");
        else {
            System.out.println("skipped         ");
        }
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        if (!isChannelAllowed("Ajara TV")) {
            System.out.println("Loading file... skipped");
            return;
        }
        int i;
        String when;
        while (true) {
            Calendar cal = Calendar.getInstance(tz);
            if (basedate == 0) {
                tr.OpenHTTP("http://www.ajaratv.com/schedulep_ru.php", null);
                when = "today";
            } else {
                cal.setTime(new Date(basedate + 24L * 60 * 60 * 1000));
                when = "" + cal.get(Calendar.DAY_OF_MONTH) + "-" + (cal.get(Calendar.MONTH) + 1) + "-" + cal.get(Calendar.YEAR);
                tr.OpenHTTP("http://www.ajaratv.com/schedulep_ru.php?show=" + when, null);
            }
            System.out.print("Loading " + when + "... ");
            tr.setAutoPrint(TVProgram.numPage);
            try {
                if (parse(tr, list))
                    System.out.println("done         ");
                else {
                    System.out.println("skipped         ");
                    break;
                }
            } finally {
                tr.Close();
            }
            if (oldbasedate == basedate) break;
            oldbasedate = basedate;
        }
    }

    public void getList(Transport tr) {
        System.out.println("Ajara TV");
    }
}

