/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:01 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.text.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPalbadpua extends tvSuperP {
    protected Pattern reDate,reChannel,reProgram;
    protected TimeZone tz;

    tvPalbadpua() {
        tz = null;
        try {
            reDate = Pattern.compile("^<title>(.+),\\s+(\\d{1,2})\\s+(.+)</title>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannel = Pattern.compile("<b><a name=\".+\"></a>(.+)</b>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgram = Pattern.compile("^\\d{1,2}\\.\\d{2}", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPalbadpua : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("albadpua.TimeZone", "Europe/Kiev"));
        setAllowedChannels(p, "albadpua");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    protected boolean parse(boolean doList, Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        String str;
        long basedate = 0;
        Matcher m;

        while ((str = tr.readLine()) != null) {
            if ((m = reDate.matcher(str)).find()) {
                basedate = TimeEngine.parseDate(m.group(2), m.group(3), null, m.group(1));
                break;
            }
        }

        ParseOneChannel currcp = null;
        boolean result = false;

        while ((str = tr.readLine()) != null) {
            if ((m = reChannel.matcher(str)).find()) {
                if (!doList) {
                    currcp = new ParseOneChannel(m.group(1));
                    list.findcreate("albadpua").add(currcp);
                } else
                    System.out.println(m.group(1));
            } else if ((m = reProgram.matcher(str)).find() && currcp != null) {
                String prgs[] = str.split("<br>");

                for (int i = 0; i < prgs.length; i++) {
                    currcp.parse(prgs[i], basedate, tz);
                }
                result = true;
            }
        }

        return result;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file... ");
        if (parse(false, tr, list))
            System.out.println("done         ");
        else
            System.out.println("skipped         ");
    }

    public void realLoad(Transport tr, ParseAllChannels list, boolean doList)
            throws IOException, ProgException, PTimeException {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumIntegerDigits(2);

        Calendar cal = Calendar.getInstance();
        boolean res;

        while (true) {
            String url = "http://www.tv.alba.dp.ua/" + nf.format(cal.get(Calendar.MONTH) + 1) + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + "u.htm";
            if (!doList) System.out.print("Loading " + nf.format(cal.get(Calendar.MONTH) + 1) + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + "... ");
            tr.OpenHTTP(url, null);
            try {
                if (!doList) tr.setAutoPrint(TVProgram.numPage);
                if (res = parse(doList, tr, list)) {
                    if (!doList) System.out.println("done         ");
                } else {
                    if (!doList) System.out.println("skipped         ");
                }
            } finally {
                tr.Close();
            }
            if (!res || doList) break;
            cal.add(Calendar.DAY_OF_MONTH, 1);
        }
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        realLoad(tr, list, false);
    }

    public void getList(Transport tr)
            throws IOException {
        try {
            realLoad(tr, null, true);
        } catch (ProgException p) {
        } catch (PTimeException p) {
        }
    }
}
