/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:01 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPcaravankz extends tvSuperP {
    protected Pattern reDate,reChannel,reProgram,reProgram2;
    protected TimeZone tz;

    tvPcaravankz() {
        tz = null;
        try {
            reDate = Pattern.compile("^<br>(\\p{L}+),\\s*(\\d{1,2})\\s+(\\p{L}+)</p>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannel = Pattern.compile("^<td class=min.+>(.+)</td>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgram = Pattern.compile("^\\d{1,2}\\.\\d{2} ");
            reProgram2 = Pattern.compile("^<td class=light.+>\\d{1,2}\\.\\d{2} ");
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPcaravankz : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("caravankz.TimeZone", "Asia/Almaty"));
        setAllowedChannels(p, "caravankz");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    protected boolean parse(boolean doList, Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        boolean result = false;
        String str;
        Matcher m;
        long basedate = 0;

        while ((str = tr.readLine()) != null) {
            str = str.trim();
            if ((m = reDate.matcher(str)).find()) {
                basedate = TimeEngine.parseDate(m.group(2), m.group(3), null, m.group(1));
                break;
            }
        }
        if (basedate == 0) return false;

        ParseOneChannel currcp = null;

        while ((str = tr.readLine()) != null) {
            str = str.trim();
            if ((m = reChannel.matcher(str)).find()) {
                currcp = new ParseOneChannel(m.group(1));
                list.findcreate("caravankz").add(currcp);
            } else if ((m = reProgram.matcher(str)).find() && basedate != 0 && currcp != null) {
                str = StringUtils.replaceAllCase(str, "</td>", "");
                str = StringUtils.replaceAllCase(str, "<br>", "");
                currcp.parse(str, basedate, tz);
                result = true;
            } else if ((m = reProgram2.matcher(str)).find() && basedate != 0 && currcp != null) {
                str = StringUtils.replaceAllCase(str, "</td>", "");
                str = StringUtils.replaceAllCase(str, "<br>", "");
                int b = str.indexOf(">");
                str = str.substring(b + 1);
                currcp.parse(str, basedate, tz);
                result = true;
            } else if (str.toLowerCase().equals("</table>")) {
                break;
            }
        }
        return result;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        boolean res = false;
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file... ");
        while (!tr.eof())
            if (parse(false, tr, list)) res = true;
        if (res)
            System.out.println("done         ");
        else
            System.out.println("skipped      ");
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        String days[] = {"pn", "vt", "sr", "cht", "pt", "sb", "vs"};
        for (int i = 0; i < days.length; i++) {
            System.out.print("Loading local " + (i + 1) + "/" + days.length + "... ");
            tr.OpenHTTP("http://www.caravan.kz/tv/" + days[i] + ".asp", null);
            tr.setAutoPrint(TVProgram.numPage);
            try {
                if (parse(false, tr, list))
                    System.out.println("done         ");
                else
                    System.out.println("skipped         ");
            } finally {
                tr.Close();
            }
        }
        for (int i = 0; i < days.length; i++) {
            System.out.print("Loading sat " + (i + 1) + "/" + days.length + "... ");
            tr.OpenHTTP("http://www.caravan.kz/tv/" + days[i] + "s.asp", null);
            tr.setAutoPrint(TVProgram.numPage);
            try {
                if (parse(false, tr, list))
                    System.out.println("done         ");
                else
                    System.out.println("skipped         ");
            } finally {
                tr.Close();
            }
        }
    }

    public void getList(Transport tr)
            throws IOException {
        tr.OpenHTTP("http://vm.agava.ru/archive/tel/week/day/1.html", null);
        try {
            parse(true, tr, null);
        } catch (ProgException e) {
        } catch (PTimeException e) {
        } finally {
            tr.Close();
        }
    }
}
