/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:59 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPcitycat extends tvSuperP {
    protected Pattern reDate,reDateC,reChannel,reChannelInt,reProgram,reProgramEnd,reBeginHead,reEndHead,reBegin,reEnd,reH0,reH1,reH2,reSpace;
    protected TimeZone tz;

    tvPcitycat() {
        tz = null;
        try {
            reDate = Pattern.compile("^<TD\\s+ALIGN=.+VALIGN=.+><INPUT\\s+TYPE=checkbox\\s+NAME=mondat\\s+VALUE=(\\d+)>(\\d{1,2})<BR>(\\p{L}+)</TD>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reDateC = Pattern.compile("^<TD\\s+ALIGN=.+VALIGN=.+><INPUT\\s+TYPE=checkbox\\s+NAME=mondat\\s+VALUE=(\\d+)\\s+CHECKED><.+>(\\d{1,2})<BR>(\\p{L}+)</font>\\s*</TD>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannel = Pattern.compile("^<INPUT\\s+TYPE=checkbox\\s+NAME=chnl\\s+VALUE=(.+)><img.+>-(.+)<BR>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannelInt = Pattern.compile("^(.+?)\\s*<FONT.+>(.+)</FONT>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reBeginHead = Pattern.compile("<P>.+<B>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reEndHead = Pattern.compile("</B>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reBegin = Pattern.compile("<PRE>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reEnd = Pattern.compile("</PRE>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reH0 = Pattern.compile("^(\\p{L}+)\\.$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reH1 = Pattern.compile("^(\\d{1,2})\\s+(\\p{L}+)\\.$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reH2 = Pattern.compile("^(.+)$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgram = Pattern.compile("^\\d{1,2}\\.\\d{2}[\\s|,]\\s*", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPcitycat : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("citycat.TimeZone", "Europe/Moscow"));
        setAllowedChannels(p, "citycat");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    public boolean parse(Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        Matcher m0,m1,m2;
        String str;
        boolean found = false,res = false;
        Vector headers = new Vector();

        while (true) {
            while ((str = tr.readLine()) != null) {
                if (reBeginHead.matcher(str.trim()).find()) {
                    found = true;
                    break;
                }
            }
            if (!found || str == null) return res;
            headers.clear();
            while ((str = tr.readLine()) != null) {
                if (reEndHead.matcher(str.trim()).find()) {
                    found = true;
                    break;
                }
                headers.add(str);
            }
            if (!found || str == null) return false;
            if (headers.size() != 3) throw new ProgException("Bad date format");
            if (!(m0 = reH0.matcher((String) headers.elementAt(0))).find() || !(m1 = reH1.matcher((String) headers.elementAt(1))).find() || !(m2 = reH2.matcher((String) headers.elementAt(2))).find()) throw new ProgException("Bad date format");

            String dayofweek,daynum,month,cname;
            dayofweek = m0.group(1);
            daynum = m1.group(1);
            month = m1.group(2);
            cname = m2.group(1);

//        if (!isChannelAllowed(cname)) return false;
            ParseOneChannel currcp = new ParseOneChannel(cname);
            long basedate = TimeEngine.parseDate(daynum, month, null, dayofweek);
            list.findcreate("citycat").add(currcp);
            while ((str = tr.readLine()) != null) {
                str = str.trim();
                if (str.endsWith(".")) str = str.substring(0, str.length() - 1);
                if (reEnd.matcher(str).find()) break;
                if (reProgram.matcher(str).find()) {
                    currcp.parse(str, basedate, tz);
                    res = true;
                } else { // Анонс
                    currcp.addAnnouncement(str);
                }//throw new ProgException("Error in line #"+tr.getCurrentLineNum()+": bad line");
            }
        }
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        boolean res = false;
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file...");
        while (!tr.eof())
            if (parse(tr, list)) res = true;
        if (res)
            System.out.println("done         ");
        else
            System.out.println("skipped      ");
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        Vector dateList = new Vector(),channelList = new Vector(),channelNames = new Vector();

        System.out.print("Loading list...");
        realLoad(false, tr, dateList, channelList, channelNames);
        System.out.println("done");
        int i,j;

        String adate = "",achan = "";

        for (i = 0; i < dateList.size(); i++)
            adate += "+" + dateList.elementAt(i);
        for (j = 0; j < channelList.size(); j++)
            if (j == 0)
                achan = "" + channelList.elementAt(j);
            else
                achan += "+" + channelList.elementAt(j);

        System.out.print("Loading...");
        tr.OpenHTTP("http://www.citycat.ru/tv/tv.cgi", "mondat=" + adate + "&thief=Ya+xochu+ukrast+vashu+programmu&chnl_all=" + achan + "&chnl=" + achan);
        tr.setAutoPrint(TVProgram.numPage);
        try {
            if (parse(tr, list))
                System.out.println("done         ");
            else
                System.out.println("skipped      ");
        } finally {
            tr.Close();
        }
    }

    public void getList(Transport tr)
            throws IOException {
        realLoad(true, tr, null, null, null);
    }

    protected void realLoad(boolean doList, Transport tr, Vector dateList, Vector channelList, Vector channelNames)
            throws IOException {
        Matcher m,m2;
        String cname;

        tr.OpenHTTP("http://www.citycat.ru/tv/tv.cgi", null);
        try {
            String str;
            while ((str = tr.readLine()) != null) {
                str = str.trim();
                if ((m = reDate.matcher(str)).find() && !doList) {
                    dateList.add(m.group(1));
                } else if ((m = reDateC.matcher(str)).find() && !doList) {
                    dateList.add(m.group(1));
                } else if ((m = reChannel.matcher(str)).find()) {
                    if ((m2 = reChannelInt.matcher(m.group(2))).find()) {
                        cname = m2.group(1) + m2.group(2);
                    } else
                        cname = m.group(2);
                    if (doList) {
                        System.out.println(cname);
                    } else if (isChannelAllowed(cname)) {
                        channelList.add(m.group(1));
                        channelNames.add(cname);
                    }
                }
            }
        } finally {
            tr.Close();
        }
    }
}
