/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.2 $
 *     $Date: 2004/04/22 11:17:04 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPcosmostv extends tvSuperP {
    protected Pattern reFile,reChannel,reChannelAnn,reProgram;
    protected TimeZone tz;

    tvPcosmostv() {
        tz = null;
        try {
            reFile = Pattern.compile("^<a\\s+href=.*schedule/(.+\\.txt)[\"|\']?>.+</a>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannel = Pattern.compile("^(\\d{2})\\.(\\d{2})\\.(\\d{4})\\s+(.+)$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannelAnn = Pattern.compile("^(\\d{2})\\.(\\d{2})\\.(\\d{4})\\s+\\(Анонс\\)\\s+(.+)$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgram = Pattern.compile("^(\\d{2}:\\d{2})\\s+(.+)$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPcosmostv : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("cosmostv.TimeZone", "Europe/Minsk"));
        setAllowedChannels(p, "cosmostv");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    protected boolean addann(OneSite site, String channelName, long currtime, String ann) {
        boolean found = false;
        ParseProg p;
        ParseOneChannel ch;
        for (int m = 0; m < site.size(); m++) {
            ch = (ParseOneChannel) site.elementAt(m);
            if (channelName.equals(ch.ChannelName)) {
                p = ch.findByTime(currtime);
                if (p != null) {
                    p.addAnnouncement(ann);
                    found = true;
                    break;
                }
            }
        }
        if (found) return true;
        for (int m = 0; m < site.size(); m++) {
            ch = (ParseOneChannel) site.elementAt(m);
            if (channelName.equals(ch.ChannelName)) {
                p = ch.findByTime(currtime - 24L * 60 * 60 * 1000);
                if (p != null) {
                    p.addAnnouncement(ann);
                    found = true;
                    break;
                }
            }
        }
        return found;
    }

    protected boolean parse(boolean doList, Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        String str;

        if ((str = tr.readLine()) != null) {
            if (!str.equals("C O S M O S T V . C O M"))
                return false;
        } else
            return false;

        boolean result = false;
        boolean ann = false;
        Matcher m;
        String daynum = null,month = null,year = null;
        long basedate = 0,curtime;
        TreeSet listp = new TreeSet();
        ParseOneChannel currcp = null;
        OneSite site = list.findcreate("cosmostv");

        while ((str = tr.readLine()) != null) {
            if ((m = reChannelAnn.matcher(str)).find()) {
                if (!doList) {
                    currcp = (ParseOneChannel) site.find(m.group(4));
                    ann = true;
                    daynum = m.group(1);
                    month = m.group(2);
                    year = m.group(3);
                    basedate = TimeEngine.parseDate(daynum, month, year, null);
                }
            } else if ((m = reChannel.matcher(str)).find()) {
                if (!doList) {
                    currcp = new ParseOneChannel(m.group(4));
                    ann = false;
                    site.add(currcp);
                    daynum = m.group(1);
                    month = m.group(2);
                    year = m.group(3);
                    basedate = TimeEngine.parseDate(daynum, month, year, null);
                } else
                    listp.add(m.group(4));
            } else if ((m = reProgram.matcher(str)).find() && currcp != null) {
                curtime = TimeEngine.parseTime(m.group(1), tz, basedate) + basedate;
                if (!ann) {
                    currcp.add(new ParseProg(true, curtime, 0, m.group(2)));
                    result = true;
                } else {
                    if (!addann(site, currcp.ChannelName, curtime, m.group(2)))
                        System.out.println("Can't find base for announce " + (new Date(curtime)).toString() + "/" + m.group(2));
                }
            }
        }

        if (doList) {
            Iterator it = listp.iterator();
            while (it.hasNext())
                System.out.println(it.next());
        }
        return result;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file... ");
        if (parse(false, tr, list))
            System.out.println("done         ");
        else
            System.out.println("skipped         ");
    }

    public void realLoad(Transport tr, ParseAllChannels list, boolean doList)
            throws IOException, ProgException, PTimeException {
        Matcher m;
        String str;
        HashSet li = new HashSet();

        if (!doList) System.out.print("Loading list... ");
        tr.OpenHTTP("http://www.cosmostv.com/schedule.asp", null);
        try {
            while ((str = tr.readSmallBlock("<a href=", "</a>", "", null)) != null) {
                if ((m = reFile.matcher(str)).find()) {
                    li.add(m.group(1));
                    if (doList) break;
                }
            }
        } finally {
            tr.Close();
        }
        if (!doList) System.out.println("done");

        Iterator it = li.iterator();
        while (it.hasNext()) {
            str = (String) it.next();
            if (!doList) System.out.print("Loading " + str + "... ");
            tr.OpenHTTP("http://www.cosmostv.com/schedule/" + str, null);
            if (!doList) tr.setAutoPrint(TVProgram.numPage);
            try {
                if (parse(doList, tr, list)) {
                    if (!doList) System.out.println("done         ");
                } else {
                    if (!doList) System.out.println("skipped         ");
                }
            } finally {
                tr.Close();
            }
        }
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        realLoad(tr, list, false);
    }

    public void getList(Transport tr)
            throws IOException {
        try {
            realLoad(tr, null, true);
        } catch (ProgException p) {
        } catch (PTimeException p) {
        }
    }
}
