/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.4 $
 *     $Date: 2003/10/25 07:19:54 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.zaval.atv.PTimeException;
import org.zaval.atv.ParseAllChannels;
import org.zaval.atv.ParseOneChannel;
import org.zaval.atv.ParseProg;
import org.zaval.atv.ProgException;
import org.zaval.util.format.StringUtils;
import org.zaval.util.format.TimeEngine;
import org.zaval.util.io.AProperties;

public class tvPeurotv extends tvSuperP {
  protected Pattern reChannels,reDataChannelName,reDataDate,reAnnBeg,reAnnEnd,reDateUrl,reProg;
  protected TimeZone tz;

  tvPeurotv() {
    tz = null;
    try {
      reChannels = Pattern.compile("<A HREF=\"(EutvProg_reroute.cfm\\?p=.+?)\"\\s*>(.+)</a>.+\\((.+)\\)", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
      reDateUrl = Pattern.compile("^<A\\s+HREF=\"(/scripts/TVStationHome\\.cfm\\?channel=.+?&date=.+?)\">", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
      reDataChannelName = Pattern.compile("<TITLE>.+?:(.+)</TITLE>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
      reDataDate = Pattern.compile("<p>(\\w+) (\\d{1,2}) (\\w+) (\\d{4})$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
      reProg = Pattern.compile("^(\\d{2}:\\d{2})\\s+<strong>(.+)</strong>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
      reAnnBeg = Pattern.compile("^<div style=\"text-align: justify\">$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
      reAnnEnd = Pattern.compile("^</div>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
    } catch (PatternSyntaxException e) {
      System.err.println("Error create RE for tvPeurotv : " + e.getMessage());
      System.exit(1);
    }
  }

  public void setProperties(AProperties p, Transport tr) {
    tz = TimeZone.getTimeZone(p.getProperty("eurotv.TimeZone", "Europe/EST"));
    setAllowedChannels(p, "eurotv");
    tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "en");
    tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "iso-8859-1");
    tr.setCharset("ISO-8859-1");
  }

  protected boolean parse(Transport tr, ParseAllChannels list) throws ProgException, IOException, PTimeException {
    List lines = new ArrayList();
    String str;
    while ((str = tr.readLine()) != null) {
      lines.add(str.trim());
    }

    Matcher m;
    ParseOneChannel currcp = null;
    long basedate = 0;

    // find channel name and date
    for (int i = 0; i < lines.size(); i++) {
      str = (String) lines.get(i);
      if ((m = reDataChannelName.matcher(str)).find()) {
        currcp = new ParseOneChannel(m.group(1));
        list.findcreate("eurotv").add(currcp);
      } else if ((m = reDataDate.matcher(str)).find()) {
        basedate = TimeEngine.parseDate(m.group(2), m.group(3), m.group(4), m.group(1));
      }
    }

    if (currcp == null) {
      return false;
    }
    if (basedate == 0) {
      return false;
    }

    ParseProg p = null;
    boolean announce = false;

    // find programs
    for (int i = 0; i < lines.size(); i++) {
      str = (String) lines.get(i);
//      if (!"".equals(str)) {
//        System.out.println(str);
//      }
      if ((m = reProg.matcher(str)).find()) {
        long tm = TimeEngine.parseTime(m.group(1), tz, basedate);
        currcp.add(p = new ParseProg(true, tm + basedate, 0, m.group(2)));
      } else if ((m = reAnnBeg.matcher(str)).find() && p != null) {
        announce = true;
      } else if ((m = reAnnEnd.matcher(str)).find() && p != null) {
        announce = false;
      } else if (announce && p != null) {
        p.addAnnouncement(StringUtils.replaceAllCase(str, "<br>", "\n"));
      }
    }
    return true;
  }

  public void loadFromFile(Transport tr, ParseAllChannels list) throws IOException, ProgException, PTimeException {
    tr.setAutoPrint(TVProgram.numPage);
    System.out.print("Loading file... ");
    if (parse(tr, list))
      System.out.println("done         ");
    else
      System.out.println("skipped         ");
  }

  public void loadFromWWW(Transport tr, ParseAllChannels list) throws IOException, ProgException, PTimeException {
    HashMap clist = new HashMap();
    System.out.print("Loading list... ");
    realList(false, tr, clist);
    System.out.println("done         ");

    Iterator itCh = clist.keySet().iterator();
    int j = 1;
    while (itCh.hasNext()) {
      tr.resetCookies();
      // loading cookie
      tr.openHTTPfake("http://www.eurotv.com/scripts/eutvprog.cfm");

      String cname = (String) itCh.next();
      List dateUrls = getDate(tr, cname, (String)clist.get(cname));
      //tr.resetCookies();
      for (int k = 0; k < dateUrls.size(); k++) {
        System.out.print("Loading day " + (k + 1) + "/9 channel " + j + "/" + clist.size() + "-" + cname + "... ");
        boolean parsed = false;
        for (int t = 0; t < 10; t++) {
            tr.OpenHTTP("http://www.eurotv.com" + (String) dateUrls.get(k), null);
            if (parse(tr, list)) {
              parsed = true;
              break;
            } else {
              try {
                System.out.print("reload... ");
                Thread.sleep(3000);
              } catch (Exception ex) {}
            }
        }
        if (parsed)
          System.out.println("done         ");
        else
          System.out.println("skipped         ");
      }
      j++;
    }
  }

  public void getList(Transport tr) throws IOException {
    realList(true, tr, null);
  }

  protected void realList(boolean doList, Transport tr, HashMap list) throws IOException {
    String str;
    Matcher m;
    tr.OpenHTTP("http://www.eurotv.com/scripts/alpha.cfm", null);
    try {
      while ((str = tr.readLine()) != null) {
        if ((m = reChannels.matcher(str)).find()) {
          if (!doList) {
            if (isChannelAllowed(m.group(2)))
              list.put(m.group(2), m.group(1));
          } else
            System.out.println(m.group(2));
        }
      }
    } finally {
      tr.Close();
    }
  }

  protected List getDate(Transport tr, String chName, String URL) throws IOException {
//    try {
      tr.OpenHTTP("http://www.eurotv.com/scripts/"+URL, null);
/*    } catch (Exception e) {
      try {
        tr.OpenHTTP("http://www.eurotv.com/sl" + chName, null);
      } catch (Exception ex) {
        return (new Vector());
      }
    }*/
    String s;
    Matcher m;
    List result = new ArrayList();
    try {
      while ((s = tr.readLine()) != null)
        if ((m = reDateUrl.matcher(s)).find())
          result.add(m.group(1));
    } finally {
      tr.Close();
    }
    return result;
  }
}
