/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:58 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPkulichki extends tvSuperP {
    protected Pattern reListMain,reListMainPartWeek,reListMainPartPackets,reListDet,reBegin,reEnd,reProgram;
    protected TimeZone tz;
    HashMap anns;
    ParseOneChannel chan;
    protected HashMap loadChannelsNames;
    protected HashMap chRegs;

    tvPkulichki() {
        loadChannelsNames = new HashMap();
        tz = null;
        try {
            reListMain = Pattern.compile("^<FORM\\s+action=cgi-bin/gpack.cgi\\s+method=post>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reListMainPartWeek = Pattern.compile("^<input.+name=\"?week\"?\\s+value=(\\d+)", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reListMainPartPackets = Pattern.compile("^<input.+name=\"?pakets\"?.+value=(\\w+)>(.+)<BR>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reListDet = Pattern.compile("^<input\\s+type=\"?checkbox\"?\\s+name=\"?chanel\"?.+value=\"?(.+?)\"?>(.+)<BR>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reBegin = Pattern.compile("<P>.*<B>(\\p{L}+)\\.\\s*(\\d{1,2})\\s+(\\p{L}+)\\.\\s*(.+?)<.+<pre>(.+)$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reEnd = Pattern.compile("</pre>(.*)$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgram = Pattern.compile("^\\d{1,2}:\\d{2}\\s+", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPkulichki : " + e.getMessage());
            System.exit(1);
        }
        anns = new HashMap();
        chan = null;
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("kulichki.TimeZone", "Europe/Moscow"));
        setAllowedChannels(p, "kulichki");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    public boolean parse(Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        String str;
        Matcher m = null;
        ParseOneChannel currcp = null;
        boolean found = false;
        long basedate = 0;

        while ((str = tr.readLine()) != null)
            if ((m = reBegin.matcher(str)).find()) {
                found = true;
                break;
            }

        if (!found || str == null) return false;

        String dayofweek,daynum,month,cname;
        dayofweek = m.group(1);
        daynum = m.group(2);
        month = m.group(3);
        cname = m.group(4);
        str = m.group(5);
        tr.pushString(str);

//        if (isChannelAllowed(cname)) {
        currcp = new ParseOneChannel(cname);
        basedate = TimeEngine.parseDate(daynum, month, null, dayofweek);
        list.findcreate("kulichki").add(currcp);
//        }

        while ((str = tr.readLine()) != null) {
            if ((m = reEnd.matcher(str)).find()) {
                tr.pushString(m.group(1));
                break;
            }
            if (currcp != null) {
                if (reProgram.matcher(str).find()) {
                    currcp.parse(str, basedate, tz);
                } else {
                    currcp.addAnnouncement(str);
                } // throw new ProgException("Error in line #"+tr.getCurrentLineNum()+": bad line");
            }
        }
        if (currcp == null)
            return false;
        else
            return currcp.size() > 0;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        boolean res = false;
        chRegs = new HashMap();
        System.out.print("Loading file...");
        tr.setAutoPrint(TVProgram.numPage);
        while (!tr.eof())
            if (parse(tr, list)) res = true;
        if (res)
            System.out.println("done         ");
        else
            System.out.println("skipped      ");
        chRegs = null;
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        String str, week;
        TreeMap nm = new TreeMap();
        chRegs = new HashMap();
        System.out.print("Loading list...");
        week = realList(false, tr, nm);
        System.out.println("done");

        for (int i = 1; i <= 7; i++) {
            Iterator itCh = nm.keySet().iterator();
            int j = 1;
            while (itCh.hasNext()) {
                String v = (String) itCh.next();
                System.out.print("Loading day " + i + "/7, channel " + j + "/" + nm.size() + "(" + nm.get(v) + ")...");

                str = "week=" + week + "&day=" + i + "&chanel=" + v + "\n";
                byte[] dat = tr.convert(str, "Cp1251");
                tr.OpenHTTP("http://tv.kulichki.net/andgon/cgi-bin/itv.cgi", new String(dat));
                tr.setAutoPrint(TVProgram.numPage);
                try {
                    if (parse(tr, list))
                        System.out.println("done         ");
                    else
                        System.out.println("skipped      ");
                } finally {
                    tr.Close();
                }
                j++;
            }
        }
        chRegs = null;
    }

    public void getList(Transport tr)
            throws IOException {
        realList(true, tr, null);
    }

    protected String realList(boolean doList, Transport tr, TreeMap nm)
            throws IOException {
        String str,week = null;
        Matcher m;
        Vector packets = new Vector();

        tr.OpenHTTP("http://tv.kulichki.net", null);
        try {
            while ((str = tr.readLine()) != null) {
                if ((m = reListMain.matcher(str)).find()) {
                    int b,e = 0;
                    while (true) {
                        b = str.toUpperCase().indexOf("<INPUT", e);
                        if (b == -1) break;
                        e = str.toUpperCase().indexOf("<INPUT", b + 1);
                        String p = str.substring(b, e);
                        if ((m = reListMainPartWeek.matcher(p)).find()) {
                            if (week != null) break;
                            week = m.group(1);
                        } else if ((m = reListMainPartPackets.matcher(p)).find()) {
                            if (!m.group(1).equals("gor") && !m.group(1).equals("anons")) {
                                packets.add(m.group(1));
                                chRegs.put(m.group(1), m.group(2));
                            }
                        }
                    }
                }
            }
        } finally {
            tr.Close();
        }

        str = "week=" + week;
        for (int i = 0; i < packets.size(); i++)
            str += "&pakets=" + packets.elementAt(i);
        if (!doList) System.out.print("+details...");
        byte[] dat = tr.convert(str, "Cp1251");
        tr.OpenHTTP("http://tv.kulichki.net/cgi-bin/gpack.cgi", new String(dat));
        try {
            while ((str = tr.readSmallBlock("<input", "<BR>", "", null)) != null) {
                if ((m = reListDet.matcher(str)).find())
                    if (doList) {
                        System.out.println(m.group(2));
                    } else {
                        if (isChannelAllowed(m.group(2)))
                            nm.put(m.group(1), m.group(2));
                    }
            }
        } finally {
            tr.Close();
        }
        return week;
    }
}
