/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:58 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPntvplus extends tvSuperP {
    protected Pattern reDate;
    protected TimeZone tz;

    tvPntvplus() {
        tz = null;
        try {
            reDate = Pattern.compile("^(\\d{1,2})\\s+(\\p{Alpha}+)\\s+(\\d{4})\\.\\s+(.+)$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPvm : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("ntvplus.TimeZone", "Europe/Moscow"));
        setAllowedChannels(p, "ntvplus");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    protected boolean parse(boolean doList, Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        boolean result = false;
        String str;
        Matcher m;
        ParseOneChannel currcp = null;

        String daynum = null,month = null,year = null;
        long basedate = 0;
        while ((str = tr.readLine()) != null) {
            if ((m = reDate.matcher(str)).find()) {
                daynum = m.group(1);
                month = m.group(2);
                year = m.group(3);
                basedate = TimeEngine.parseDate(daynum, month, year, null);
                if (doList) {
                    System.out.println(m.group(4));
                } else /*if (isChannelAllowed(m.group(4)))*/ {
                    currcp = new ParseOneChannel(m.group(4));
                    list.findcreate("ntvplus").add(currcp);
                }
            } else if (currcp != null && basedate != 0 && !doList && currcp.canParse(str)) {
                currcp.parse(str, basedate, tz);
                result = true;
            } else if (currcp != null) {
                str = str.trim();
                if (str.length() > 0)
                    currcp.addAnnouncement(str);
            }
        }
        return result;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file... ");
        if (parse(false, tr, list))
            System.out.println("done         ");
        else
            System.out.println("skipped         ");
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        System.out.print("Loading... ");
        tr.OpenHTTP("http://www.ntvplus.ru/ntvplus/schedule/ListTV.ltv", null);
        tr.setAutoPrint(TVProgram.numPage);
        try {
            if (parse(false, tr, list))
                System.out.println("done         ");
            else
                System.out.println("skipped         ");
        } finally {
            tr.Close();
        }
    }

    public void getList(Transport tr)
            throws IOException {
        tr.OpenHTTP("http://www.ntvplus.ru/ntvplus/schedule/ListTV.ltv", null);
        try {
            parse(true, tr, null);
        } catch (ProgException e) {
        } catch (PTimeException e) {
        } finally {
            tr.Close();
        }
    }
}





