/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:00:59 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.text.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPrbcru extends tvSuperP {
    protected Pattern reChannel,reProgram,reAnn;
    protected TimeZone tz;
    protected HashMap anns = null;

    tvPrbcru() {
        tz = null;
        try {
            reChannel = Pattern.compile("^(.+)</b></font><.+>(\\p{L}+),\\s+(\\d{1,2})\\s+(\\p{L}+)\\s+(\\d{4})", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgram = Pattern.compile("^<b>(\\d{1,2}:\\d{2})</b>\\s*(.+?)<", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reAnn = Pattern.compile("my=window.open\\('(http://tv.rbc.ru/.+shtml)','anonse'", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPrbcru : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("rbcru.TimeZone", "Europe/Moscow"));
        setAllowedChannels(p, "rbcru");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setCharset("Cp1251");
    }

    protected boolean parse(boolean doList, Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        ParseOneChannel currcp = null;
        ParseProg p = null;
        boolean result = false;
        String str;
        Matcher m;
        long basedate = 0,curtime;
        int b;

        while ((str = tr.readLine()) != null) {
            if ((m = reAnn.matcher(str)).find() && anns != null && p != null) {
                anns.put(m.group(1), p);
            } else if ((b = str.indexOf("</b></font>")) != -1) {//!!! bad regexp
                if ((m = reChannel.matcher(str)).find()) {
                    p = null;
                    if (!doList) {
                        currcp = new ParseOneChannel(m.group(1));
                        list.findcreate("rbcru").add(currcp);
                        basedate = TimeEngine.parseDate(m.group(3), m.group(4), m.group(5), m.group(2));
                    } else
                        System.out.println(m.group(1));
                }
            } else if ((b = str.indexOf("<b>")) != -1) {//!!! bad regexp
                str = str.substring(b);
                if ((m = reProgram.matcher(str)).find() && currcp != null) {
                    curtime = TimeEngine.parseTime(m.group(1), tz, basedate) + basedate;
                    currcp.add(p = new ParseProg(true, curtime, 0, m.group(2)));
                    result = true;
                }
            }
        }

        return result;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file... ");
        if (parse(false, tr, list))
            System.out.println("done         ");
        else
            System.out.println("skipped         ");
    }

    public void realLoad(Transport tr, ParseAllChannels list, boolean doList)
            throws IOException, ProgException, PTimeException {
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumIntegerDigits(2);

        Calendar cal;
        boolean res;

        if (!doList)
            anns = new HashMap();
        for (int i = 1; i <= 3; i++) {
            cal = Calendar.getInstance();
            while (true) {
                String url = "http://tv.rbc.ru/?d" + cal.get(Calendar.YEAR) + nf.format(cal.get(Calendar.MONTH) + 1) + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + "g" + i;
                if (!doList) System.out.print("Loading " + cal.get(Calendar.YEAR) + nf.format(cal.get(Calendar.MONTH) + 1) + nf.format(cal.get(Calendar.DAY_OF_MONTH)) + " " + i + "/3... ");
                tr.OpenHTTP(url, null);
                try {
                    if (!doList) tr.setAutoPrint(TVProgram.numPage);
                    if (res = parse(doList, tr, list)) {
                        if (!doList) System.out.println("done         ");
                    } else {
                        if (!doList) System.out.println("skipped         ");
                    }
                } finally {
                    tr.Close();
                }
                if (!res || doList) {
                    break;
                }
                cal.add(Calendar.DAY_OF_MONTH, 1);
            }
        }
        if (anns != null) {
            int i = 1;
            String url,str;
            Iterator it = anns.keySet().iterator();
            while (it.hasNext()) {
                url = (String) it.next();
                System.out.print("Loading announce " + i + "/" + anns.size() + "... ");
                tr.OpenHTTP(url, null);
                try {
                    while ((str = tr.readLine()) != null) {
                        if (str.trim().toLowerCase().equals("<font size=\"-1\">")) {
                            str = tr.readLine();
                            if (str != null)
                                ((ParseProg) anns.get(url)).addAnnouncement(str);
                            break;
                        }
                    }
                } finally {
                    tr.Close();
                }
                System.out.println("done");
                i++;
            }
        }
        anns = null;
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        realLoad(tr, list, false);
    }

    public void getList(Transport tr)
            throws IOException {
        try {
            realLoad(tr, null, true);
        } catch (ProgException p) {
        } catch (PTimeException p) {
        }
    }
}
