/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:01 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPtvtodayde extends tvSuperP {
    protected Pattern reHead,reOption,reTime,reHeadline,reText,reLink,reDate;
    protected TimeZone tz;
    protected Map data = null,times = null;

    tvPtvtodayde() {
        tz = null;
        try {
            reHead = Pattern.compile("<span\\s+class=\"text-grau\".*>(.+)</span>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reOption = Pattern.compile("^<option\\s+value=\"(.+)\".*>(.+)</option>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reDate = Pattern.compile("<span\\s+class=\"text-weiss\">(\\d{1,2})\\.(\\d{1,2})\\.(\\d{4}).*</span>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reTime = Pattern.compile("^<span\\s+class=\"text-grau\">(\\d{1,2}\\.\\d{2})</span>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reHeadline = Pattern.compile("^<span\\s+class=\"headline\">(.+)\\s*\\((.+)\\)\\s*</span>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reLink = Pattern.compile("^<span\\s+class=\"link-intern\"><u>(.+)\\s*\\((.+)\\)\\s*</u></span>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reText = Pattern.compile("^<span\\s+class=\"text\">(.+)</span>$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPtvtodayde : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("tvtodayde.TimeZone", "Europe/Berlin"));
        setAllowedChannels(p, "tvtodayde");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "en");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "iso-8859-1");
        tr.setCharset("ISO-8859-1");
    }

    public ParseProg addprog(String cname, long tm, String pname) {
        ParseOneChannel ch = null;
        ParseProg p;

        ch = (ParseOneChannel) data.get(cname);
        if (ch == null) {
            ch = new ParseOneChannel(cname);
            data.put(cname, ch);
        }

/*        Long tmo=(Long)times.get(cname);
        if (tmo!=null) {
            if (tmo.longValue()>tm) {
                tm+=24L*60*60*1000;
System.out.println("add time "+new Date(tm).toString()+" "+pname);
            }
        }
        tmo=new Long(tm);
        times.put(cname,tmo);*/

        ch.add(p = new ParseProg(true, tm, 0, pname.trim()));
        return p;
    }

    public int parse(Transport tr, ParseAllChannels list)
            throws ProgException, IOException, PTimeException {
        String str;
        ParseProg p = null;
        Matcher m;
        long tm = 0,basedate = 0;
        int co = 0;
        while ((str = tr.readLine()) != null) {
            if ((m = reDate.matcher(str)).find()) {
                basedate = TimeEngine.parseDate(m.group(1), m.group(2), m.group(3), null);
            }
            if (str.indexOf("<!--LISTING ANFANG-->") != -1) break;
        }
        if (str == null) return co;

        while ((str = tr.readSmallBlock("<span", "</span>", " ", "<!--LISTING ENDE-->")) != null) {
            if (str.equals("<!--LISTING ENDE-->")) break;
            if ((m = reTime.matcher(str)).find()) {
                co++;
                tm = TimeEngine.parseTime(m.group(1), tz, basedate) + basedate;
            } else if ((m = reHeadline.matcher(str)).find()) {
                p = addprog(m.group(2), tm, m.group(1));
            } else if ((m = reLink.matcher(str)).find()) {
                p = addprog(m.group(2), tm, m.group(1));
            } else if ((m = reText.matcher(str)).find() && p != null) {
                String dat = m.group(1).trim();
                if (!dat.equals("") && !dat.equals("<br>") && !dat.startsWith("ShowView")) {
                    if (dat.endsWith("<br>"))
                        dat = dat.substring(0, dat.length() - 4);
                    p.addAnnouncement(dat);
                }
            }
        }
        return co;
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        boolean res = false;
        data = new HashMap();
        times = new HashMap();
        tr.setAutoPrint(TVProgram.numPage);
        System.out.print("Loading file...");
        while (!tr.eof())
            if (parse(tr, list) > 0) {
                res = true;
                Iterator itCh = data.keySet().iterator();
                while (itCh.hasNext())
                    list.findcreate("tvtodayde").add((ParseOneChannel) data.get(itCh.next()));
                data.clear();
                times.clear();
            }
        if (res)
            System.out.println("done         ");
        else
            System.out.println("skipped      ");
        data = null;
        times = null;
    }

    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        Vector channelNames = new Vector();
        int first,loaded;

        System.out.print("Loading list...");
        realLoad(false, tr, channelNames);
        System.out.println("done");
        int i,j;

        data = new TreeMap();
        times = new TreeMap();
        for (i = 0; i < 7; i++) {
            for (j = 0; j < channelNames.size(); j++) {
                System.out.print("Loading program on day " + (i + 1) + "/7 for channel " + (j + 1) + "(" + channelNames.elementAt(j) + ")/" + channelNames.size() + "...");
                first = 0;
                loaded = 0;
                while (true) {
                    tr.OpenHTTP("http://www.tvtoday.de/tv/programm/programm.php?ztag=" + i + "&sender=" + new String(tr.convert((String) channelNames.elementAt(j), "ISO8859_1"), "ISO8859_1") + "&action=suche&uhrzeit=Ax06&sparte=alle&von=" + first, null);
                    tr.setAutoPrint(TVProgram.numPage);
                    try {
                        loaded = parse(tr, list);
                    } finally {
                        tr.Close();
                    }
                    if (loaded != 12) break;
                    first += 12;
                    System.out.print("+page... \b");
                }
                if (first > 0 || loaded > 0)
                    System.out.println("done         ");
                else
                    System.out.println("skipped      ");
            }
            Iterator itCh = data.keySet().iterator();
            while (itCh.hasNext())
                list.findcreate("tvtodayde").add((ParseOneChannel) data.get(itCh.next()));
            data.clear();
            times.clear();
        }
        data = null;
        times = null;
    }

    public void getList(Transport tr)
            throws IOException {
        realLoad(true, tr, null);
    }

    protected void realLoad(boolean doList, Transport tr, Vector channelNames)
            throws IOException {
        Matcher m;
        String cname;

        tr.OpenHTTP("http://www.tvtoday.de/tv/programm/Default.php", null);
        try {
            String str;
            while ((str = tr.readLine()) != null) {
                if ((m = reHead.matcher(str)).find()) {
                    if (m.group(1).equals("Sender")) break;
                }
            }
            while ((str = tr.readSmallBlock("<option value", "</option>", "", "</tr>")) != null) {
                if ((m = reOption.matcher(str)).find()) {
                    cname = m.group(1);
                    if (!cname.equals("alle") && !cname.equals("gesamt") && !cname.equals("HS") && !cname.equals("RS") && !cname.equals("HR") && !cname.equals("SS") && !cname.equals("AU")) {
                        if (doList)
                            System.out.println(cname);
                        else if (isChannelAllowed(cname))
                            channelNames.add(cname);
                    }
                }
            }
        } finally {
            tr.Close();
        }
    }
}
