/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.3 $
 *     $Date: 2004/03/31 10:44:12 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;
import java.util.regex.*;

import org.zaval.atv.*;
import org.zaval.util.format.*;
import org.zaval.util.io.*;

public class tvPvsetv extends tvSuperP {
  protected String userName = "atvrobot";
  protected String password = "atvgrab";

    protected Pattern reDate,reDateA,reBlock,reChannel,reChannelAnn,reProgram,reProgramA,reAnn,reListChannels,reListChannelsPart,reListDates,reListDatesPart;
    protected TimeZone tz;
    protected HashMap loadChannelsNames;
    OneSite site = null;

    tvPvsetv() {
        loadChannelsNames = new HashMap();
        tz = null;
        try {
            reDate = Pattern.compile("<b class=title>\\s*(\\p{L}+),\\s*(\\d{1,2})\\s+(\\p{L}+)\\s+(\\d{4})\\s*</b>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannel = Pattern.compile("<font class=white>(.+?)</font>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reChannelAnn = Pattern.compile("<font class=white>(.+?)</font>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgram = Pattern.compile("(\\d{1,2}:\\d{2})</b></td><td>(.+?)</td>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reProgramA = Pattern.compile("(\\d{1,2}:\\d{2})</b></td><td>\\s*<a href=films.php\\?fid=(\\d)+.+?>(.+?)</a>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reAnn = Pattern.compile("(\\d{1,2}:\\d{2})</b><br><b class=blue>(.+?)</b>(.+?)</td>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reListChannels = Pattern.compile("<input type=checkbox name=cch(\\d+)>(.+?)<", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
            reListDates = Pattern.compile("<option value=(\\d{4}-\\d{2}-\\d{2}).*>.+?</option>", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for tvPvsetv : " + e.getMessage());
            System.exit(1);
        }
    }

    public void setProperties(AProperties p, Transport tr) {
        tz = TimeZone.getTimeZone(p.getProperty("vsetv.TimeZone", "Europe/London"));
        setAllowedChannels(p, "vsetv");
        tr.setHeader(Transport.HEADER_ACCEPT_LANGUAGE, "ru");
        tr.setHeader(Transport.HEADER_ACCEPT_CHARSET, "windows-1251");
        tr.setHeader(Transport.HEADER_REFERER, "http://www.vsetv.com");
        tr.setCharset("Cp1251");
    }

    public boolean parse(Transport tr)
            throws ProgException, IOException, PTimeException {
        boolean result = false;
        Matcher m;
        String str;
        long basedate = 0;
        ParseOneChannel chan = null;

        loadChannelsNames.clear();
        chan = null;
        while ((str = tr.readLine()) != null) {
            if ((m = reDate.matcher(str)).find()) {
                String dayofweek,daynum,month,year;
                dayofweek = m.group(1);
                daynum = m.group(2);
                month = m.group(3);
                year = m.group(4);
                basedate = TimeEngine.parseDate(daynum, month, year, dayofweek);
            }
            if ((m = reChannel.matcher(str)).find()) {
                chan = new ParseOneChannel(m.group(1));
                site.add(chan);
                loadChannelsNames.put(chan.ChannelName, chan);
            }
            if (str.indexOf("<b class=red>") != -1 && chan != null) {
                String ps[] = str.split("<b class=red>");
                for (int i = 0; i < ps.length; i++) {
                    if ((m = reProgramA.matcher(ps[i])).find()) {
                        long tm = TimeEngine.parseTime(m.group(1), tz, basedate) + basedate;
                        chan.add(new ParseProg(true, tm, 0, m.group(3)));
//System.err.println("see announce : chan="+chan.ChannelName+"="+(new Date(tm)).toString()+ps[i]);
                        result = true;
                    } else if ((m = reProgram.matcher(ps[i])).find()) {
                        long tm = TimeEngine.parseTime(m.group(1), tz, basedate) + basedate;
                        chan.add(new ParseProg(true, tm, 0, m.group(2)));
                        result = true;
                    }
                }
            }
        }
        return result;
    }

    public void parseann(Transport tr)
            throws ProgException, IOException, PTimeException {
        Matcher m;
        String str;
        long basedate = 0;
        OneChannel chan = null;

        chan = null;
        while ((str=tr.readLine()) != null) {
            if ((m = reDate.matcher(str)).find()) {
                String dayofweek,daynum,month,year;
                dayofweek = m.group(1);
                daynum = m.group(2);
                month = m.group(3);
                year = m.group(4);
                basedate = TimeEngine.parseDate(daynum, month, year, dayofweek);
            } else if ((m = reChannelAnn.matcher(str)).find()) {
                chan = (ParseOneChannel) loadChannelsNames.get(m.group(1));
            } else if (str.indexOf("<b class=red>") != -1 && chan != null) {
                String ps[] = str.split("<b class=red>");
                for (int i = 0; i < ps.length; i++) {
                  if ((m = reAnn.matcher(ps[i])).find()) {
                    long tm = TimeEngine.parseTime(m.group(1), tz, basedate) + basedate;
                    //chan = (ParseOneChannel) loadChannelsNames.get(m.group(2));
                    if (chan == null) {
                      System.err.println("Unknown channel in announce " + str);
                      break;
                    }
                    ParseProg p = (ParseProg) chan.find(tm);
                    if (p == null) {
                      System.err.println("Unknown announce : " + (new Date(tm)).toString() + ps[i]);
                      break;
                    }
                    String ann = m.group(3).replaceAll("<.+?>", "\n").trim();
                    ann = ann.replaceAll("\n\n", "\n");
                    ann = ann.replaceAll("\n\n", "\n");
                    ann = ann.replaceAll("\n\n", "\n");
                     p.addAnnouncement(ann);
                }
              }
            }
        }
        loadChannelsNames.clear();
    }

    public void loadFromFile(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {
        site = list.findcreate("vsetv");
        System.out.print("Loading file...");
        tr.setAutoPrint(TVProgram.numPage);
        while (!tr.eof())
            parse(tr);
        System.out.println("done         ");
    }

  TreeSet dates = new TreeSet();

  protected void login(Transport tr) throws IOException {
    tr.openHTTP("http://www.vsetv.com");
    tr.OpenHTTP("http://www.vsetv.com/login.php",
                "uid=" + tr.getCookie("uid") +
                "&inlogin=" + userName +
                "&fromscript=%2F" +
                "&inpassword=" + password +
                "&x=16&y=10");
    Pattern dp = Pattern.compile(".+value=(\\d{4}-\\d{2}-\\d{2}).+");
    String str;
    while ((str = tr.readLine()) != null) {
      while (true) {
        int i = str.indexOf('>',1);
        if (i == -1) break;
        Matcher m = dp.matcher(str.substring(0, i));
        if (m.matches()) {
          dates.add(m.group(1));
        }
        str = str.substring(i);
      }
    }
  }

  protected void load(Transport tr)
      throws IOException, ProgException, PTimeException {
    Iterator it = dates.iterator();
    int i=1;
    while (it.hasNext()) {
      String dt = (String)it.next();
      System.out.print("Load "+dt+"("+i+"/"+dates.size()+")...");
      tr.openHTTP("http://www.vsetv.com/vsetv.php" +
                  "?category=prog" +
                  "&selectsort=chan" +
                  "&selectdate="+dt +
                  "&hours1=5" +
                  "&hours2=5" +
                  "&selectchannels=personal" +
                  "&uid="+tr.getCookie("uid")+
                  "&x=17&y=5");
      tr.setAutoPrint(TVProgram.numPage);
      parse(tr);
      System.out.println("+ann...");
      tr.openHTTP("http://www.vsetv.com/vsetv.php" +
                  "?category=anfi" +
                  "&selectsort=chan" +
                  "&selectdate=" + dt +
                  "&hours1=5" +
                  "&hours2=5" +
                  "&selectchannels=personal" +
                  "&uid=" + tr.getCookie("uid") +
                  "&x=20&y=1");
      tr.setAutoPrint(TVProgram.numPage);
      parseann(tr);
      System.out.println("done");
      i++;
    }
  }


    public void loadFromWWW(Transport tr, ParseAllChannels list)
            throws IOException, ProgException, PTimeException {

      site = list.findcreate("vsetv");

      login(tr);
      load(tr);
    }

    public void getList(Transport tr)
            throws IOException {
        realLoadList(true, tr, null, null);
    }

    protected void realLoadList(boolean doList, Transport tr, TreeSet loadChannels, TreeSet loadDates)
            throws IOException {
        tr.OpenHTTP("http://www.vsetv.com", null);
        try {
            String str;
            Matcher m;
            while ((str = tr.readLine()) != null) {
                if ((m = reListChannels.matcher(str)).find()) {
                    while (true) {
                        int p = str.indexOf("<input type=checkbox name=");
                        if (p == -1) break;
                        str = str.substring(p);
                        if ((m = reListChannels.matcher(str)).find()) {
                            if (!doList)
                                loadChannels.add(new Integer(m.group(1)));
                            else
                                System.out.println(m.group(2));
                        }
                        str = str.substring(1);
                    }
                } else if ((m = reListDates.matcher(str)).find()) {
                    while (true) {
                        int p = str.indexOf("<option value=");
                        if (p == -1) break;
                        str = str.substring(p);
                        if ((m = reListDates.matcher(str)).find()) {
                            loadDates.add(m.group(1));
                        }
                        str = str.substring(1);
                    }
                }
            }
        } finally {
            tr.Close();
        }
    }
}
