/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.2 $
 *     $Date: 2004/03/31 10:44:12 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.atv.loader;

import java.util.*;
import java.io.*;

import org.zaval.atv.*;
import org.zaval.util.io.*;

public abstract class tvSuperP {
    public HashSet ChannelsInclude = null,ChannelsExclude = null;

    public abstract void setProperties(AProperties tvp, Transport tr);

    public abstract void loadFromFile(Transport tr, ParseAllChannels list) throws IOException, ProgException, PTimeException;

    public abstract void loadFromWWW(Transport tr, ParseAllChannels list) throws IOException, ProgException, PTimeException;

    public abstract void getList(Transport tr) throws IOException;

    public void setAllowedChannels(AProperties p, String moduleName) {
        ChannelsInclude = new HashSet();
        ChannelsExclude = new HashSet();

        String list,one;
        list = p.getProperty("list.include");
        if (list != null) {
            StringTokenizer stk = new StringTokenizer(list, ";");
            while (stk.hasMoreTokens())
                ChannelsInclude.add(stk.nextToken());
        }
        list = p.getProperty("list.exclude");
        if (list != null) {
            StringTokenizer stk = new StringTokenizer(list, ";");
            while (stk.hasMoreTokens())
                ChannelsExclude.add(stk.nextToken());
        }

        list = p.getProperty(moduleName + ".include");
        if (list != null) {
            StringTokenizer stk = new StringTokenizer(list, ";");
            while (stk.hasMoreTokens()) {
                one = stk.nextToken();
                ChannelsExclude.remove(one);
                ChannelsInclude.add(one);
            }
        }
        list = p.getProperty(moduleName + ".exclude");
        if (list != null) {
            StringTokenizer stk = new StringTokenizer(list, ";");
            while (stk.hasMoreTokens()) {
                one = stk.nextToken();
                ChannelsInclude.remove(one);
                ChannelsExclude.add(one);
            }
        }

        if (ChannelsInclude.isEmpty()) ChannelsInclude = null;
        if (ChannelsExclude.isEmpty()) ChannelsExclude = null;
    }

    public boolean isChannelAllowed(String channelName) {
        if (ChannelsExclude != null)
            if (ChannelsExclude.contains(channelName)) return false;
        if (ChannelsInclude == null) return true;
        return ChannelsInclude.contains(channelName);
    }

  protected void dumpPage(String filename, Transport tr, String charset) throws IOException {
    BufferedOutputStream f=new BufferedOutputStream(new FileOutputStream(filename));
    String str;
    while ((str = tr.readLine()) != null) {
      f.write(str.getBytes(charset));
      f.write("\n".getBytes(charset));
    }
    f.flush();
    f.close();
  }
}
