/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:02 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.util.format;

import java.util.*;
import java.util.regex.*;

public class TimeEngine {
    public static TimeZone localTZ = null;
    public static ArrayList monthes = null;

    public static int getMonth(String monthName) {
        if (monthes == null) {
            HashSet cm;
            monthes = new ArrayList();
            cm = new HashSet();
            cm.add("января");
            cm.add("январь");
            cm.add("gennaio");
            cm.add("januar");
            cm.add("january");
            cm.add("januaro");
            cm.add("januari");
            cm.add("janvier");
            cm.add("enero");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("февраля");
            cm.add("февраль");
            cm.add("february");
            cm.add("februar");
            cm.add("février");
            cm.add("febbraio");
            cm.add("februaro");
            cm.add("februari");
            cm.add("febrero");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("марта");
            cm.add("март");
            cm.add("march");
            cm.add("märz");
            cm.add("mars");
            cm.add("marzo");
            cm.add("marto");
            cm.add("maart");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("апреля");
            cm.add("апрель");
            cm.add("april");
            cm.add("avril");
            cm.add("aprile");
            cm.add("aprilo");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("мая");
            cm.add("май");
            cm.add("may");
            cm.add("mai");
            cm.add("maggio");
            cm.add("mayo");
            cm.add("mei");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("июня");
            cm.add("июнь");
            cm.add("june");
            cm.add("juni");
            cm.add("juin");
            cm.add("giunio");
            cm.add("junio");
            cm.add("juni");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("июля");
            cm.add("июль");
            cm.add("july");
            cm.add("juli");
            cm.add("juillet");
            cm.add("luglio");
            cm.add("julio");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("августа");
            cm.add("август");
            cm.add("august");
            cm.add("augustus");
            cm.add("août");
            cm.add("agosto");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("сентября");
            cm.add("сентябрь");
            cm.add("september");
            cm.add("septembre");
            cm.add("settembre");
            cm.add("septembro");
            cm.add("september");
            cm.add("septiembre");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("октября");
            cm.add("октябрь");
            cm.add("october");
            cm.add("octobre");
            cm.add("ottobre");
            cm.add("oktobro");
            cm.add("oktober");
            cm.add("octubre");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("ноября");
            cm.add("ноябрь");
            cm.add("november");
            cm.add("novembre");
            cm.add("novembro");
            cm.add("noviembre");
            monthes.add(cm);
            cm = new HashSet();
            cm.add("декабря");
            cm.add("декабрь");
            cm.add("december");
            cm.add("dezember");
            cm.add("décembre");
            cm.add("dicembre");
            cm.add("decembro");
            cm.add("diciembre");
            monthes.add(cm);
        }

        String[] r;
        int i;
        try {
            i = Integer.parseInt(monthName);
            if (i >= 1 && i <= 12) return i - 1;
        } catch (NumberFormatException e) {
        }

        String str = monthName.toLowerCase();
        for (int j = 0; j < monthes.size(); j++) {
            if (((HashSet) monthes.get(j)).contains(str))
                return j;
        }
        System.err.println("Error parsing month name : " + monthName);
        return -1;
    }

    public static long parseDate(String day, String month, String year, String dow) {
        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"), Locale.getDefault());
        int iyear = 0;
        int cm = getMonth(month);
        if (cm == -1) cm = cal.get(Calendar.MONTH);
        if (year != null) {
            try {
                iyear = Integer.parseInt(year);
            } catch (NumberFormatException e) {
                year = null;
            }
        }
        if (year == null) {
            iyear = cal.get(Calendar.YEAR);
            if (cal.get(Calendar.MONTH) == 11 && cm == 0) iyear++;
        }
        try {
            cal.set(iyear, cm, Integer.parseInt(day), 0, 0, 0);
        } catch (Exception e) {
            return -1;
        }
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime().getTime();
    }

    public static long parseTime(String tm, TimeZone tz, long onDate) {
        try {
            Pattern reTime = Pattern.compile("^(\\d{1,2})[\\.|:|\\s](\\d{2})$");
            Matcher ma;
            if ((ma = reTime.matcher(tm)).find()) {
                int h = Integer.parseInt(ma.group(1));
                int m = Integer.parseInt(ma.group(2));
                if (h < 0 || h > 24 || m < 0 || m > 59) return -1;
                if (h == 24 && m > 0) return -1;
                return (h * 60 + m) * 60L * 1000 - getOffset(onDate, tz);
            }
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE for parse time: " + e.getMessage());
            System.exit(1);
        }
        return -1;
    }

    public static long getOffset(long onDate, TimeZone tz) {
        Calendar c = Calendar.getInstance();
        c.setTimeZone(tz);
        c.setTime(new Date(onDate));
        return c.get(Calendar.ZONE_OFFSET) + c.get(Calendar.DST_OFFSET);
    }
}


