/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:02 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.util.io;

import java.io.*;

public class AInputStream extends InputStream {
    InputStream in;
    boolean filemode;

    public AInputStream(InputStream s, boolean reverse) {
        in = s;
        filemode = reverse;
    }

    public void close() throws IOException {
        in.close();
    }

    public int read() throws IOException {
        return in.read();
    }

    public byte readByte() throws IOException {
        return (byte) read();
    }

    public short readShort() throws IOException {
        int ch0 = read();
        int ch1 = read();
        if ((ch0 | ch1) < 0)
            throw new EOFException();
        if (filemode) {
            return (short) ((ch0 << 8) + (ch1 << 0));
        } else {
            return (short) ((ch1 << 8) + (ch0 << 0));
        }
    }

    public int readUShort() throws IOException {
        int ch0 = read();
        int ch1 = read();
        if ((ch0 | ch1) < 0)
            throw new EOFException();
        if (filemode) {
            return ((ch0 << 8) + (ch1 << 0));
        } else {
            return ((ch1 << 8) + (ch0 << 0));
        }
    }

    public long readUInt() throws IOException {
        int ch0 = read();
        int ch1 = read();
        int ch2 = read();
        int ch3 = read();
        if ((ch0 | ch1 | ch2 | ch3) < 0)
            throw new EOFException();
        if (filemode) {
            return (((long) ch0 << 24) + (ch1 << 16) + (ch2 << 8) + (ch3 << 0));
        } else {
            return (((long) ch3 << 24) + (ch2 << 16) + (ch1 << 8) + (ch0 << 0));
        }
    }

    public int readInt() throws IOException {
        int ch0 = read();
        int ch1 = read();
        int ch2 = read();
        int ch3 = read();
        if ((ch0 | ch1 | ch2 | ch3) < 0)
            throw new EOFException();
        if (filemode) {
            return (ch0 << 24) + (ch1 << 16) + (ch2 << 8) + (ch3 << 0);
        } else {
            return (ch3 << 24) + (ch2 << 16) + (ch1 << 8) + (ch0 << 0);
        }
    }

    public long readLong() throws IOException {
        long ch0 = read();
        long ch1 = read();
        long ch2 = read();
        long ch3 = read();
        long ch4 = read();
        long ch5 = read();
        long ch6 = read();
        long ch7 = read();

        if ((ch0 | ch1 | ch2 | ch3 | ch4 | ch5 | ch6 | ch7) < 0)
            throw new EOFException();
        if (filemode) {
            return (ch0 << 56) + (ch1 << 48) + (ch2 << 40) + (ch3 << 32) + (ch4 << 24) + (ch5 << 16) + (ch6 << 8) + (ch7 << 0);
        } else {
            return (ch7 << 56) + (ch6 << 48) + (ch5 << 40) + (ch4 << 32) + (ch3 << 24) + (ch2 << 16) + (ch1 << 8) + (ch0 << 0);
        }
    }

    public String readPasString(String charset) throws IOException {
        int len = readInt();
        if (len > 0) {
            byte[] d = new byte[len];
            read(d);
            if (charset == null)
                return new String(d);
            else
                return new String(d, charset);
        }
        return null;
    }

    public String readSPasString(String charset) throws IOException {
        int len = readUShort();
        if (len > 0) {
            byte[] d = new byte[len];
            read(d);
            if (charset == null)
                return new String(d);
            else
                return new String(d, charset);
        }
        return null;
    }

    public String readSPasString0(String charset) throws IOException {
        int len = readUShort();
        if (len > 0) {
            byte[] d = new byte[len];
            read(d);
            read();
            if (charset == null)
                return new String(d);
            else
                return new String(d, charset);
        }
        return null;
    }
}
