/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:02 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.util.io;

import java.io.*;

public class AOutputStream extends OutputStream {
    OutputStream out;
    boolean filemode;

    public AOutputStream(OutputStream s, boolean reverse) {
        out = s;
        filemode = reverse;
    }

    public void close() throws IOException {
        out.close();
    }

    public void write(int v) throws IOException {
        out.write(v);
    }

    public void writeByte(int v) throws IOException {
        write(v);
    }

    public void writeShort(int v) throws IOException {
        if (filemode) {
            write((v >>> 8) & 0xFF);
            write((v >>> 0) & 0xFF);
        } else {
            write((v >>> 0) & 0xFF);
            write((v >>> 8) & 0xFF);
        }
    }

    public void writeInt(int v) throws IOException {
        if (filemode) {
            write((v >>> 24) & 0xFF);
            write((v >>> 16) & 0xFF);
            write((v >>> 8) & 0xFF);
            write((v >>> 0) & 0xFF);
        } else {
            write((v >>> 0) & 0xFF);
            write((v >>> 8) & 0xFF);
            write((v >>> 16) & 0xFF);
            write((v >>> 24) & 0xFF);
        }
    }

    public void writeUInt(long v) throws IOException {
        if (filemode) {
            write((int) (v >>> 24) & 0xFF);
            write((int) (v >>> 16) & 0xFF);
            write((int) (v >>> 8) & 0xFF);
            write((int) (v >>> 0) & 0xFF);
        } else {
            write((int) (v >>> 0) & 0xFF);
            write((int) (v >>> 8) & 0xFF);
            write((int) (v >>> 16) & 0xFF);
            write((int) (v >>> 24) & 0xFF);
        }
    }

    public void writeLong(long v) throws IOException {
        if (filemode) {
            write((int) (v >>> 56) & 0xFF);
            write((int) (v >>> 48) & 0xFF);
            write((int) (v >>> 40) & 0xFF);
            write((int) (v >>> 32) & 0xFF);
            write((int) (v >>> 24) & 0xFF);
            write((int) (v >>> 16) & 0xFF);
            write((int) (v >>> 8) & 0xFF);
            write((int) (v >>> 0) & 0xFF);
        } else {
            write((int) (v >>> 0) & 0xFF);
            write((int) (v >>> 8) & 0xFF);
            write((int) (v >>> 16) & 0xFF);
            write((int) (v >>> 24) & 0xFF);
            write((int) (v >>> 32) & 0xFF);
            write((int) (v >>> 40) & 0xFF);
            write((int) (v >>> 48) & 0xFF);
            write((int) (v >>> 56) & 0xFF);
        }
    }

    public void writePasString(String str, String charset)
            throws IOException {
        if (str == null) {
            writeInt(0);
        } else {
            byte[] sb;
            if (charset == null)
                sb = str.getBytes();
            else
                sb = str.getBytes(charset);
            int len = sb.length;
            writeInt(len);
            write(sb);
        }
    }

    public int writeSPasString(String str, String charset)
            throws IOException {
        if (str == null) {
            writeShort(0);
            return 0;
        } else {
            byte[] sb;
            if (charset == null)
                sb = str.getBytes();
            else
                sb = str.getBytes(charset);
            int len = sb.length;
            if (len > 65535) len = 65535;
            writeShort(len);
            write(sb, 0, len);
            return len;
        }
    }

    public int calcSPasString(String str, String charset)
            throws IOException {
        if (str == null) {
            return 2;
        } else {
            byte[] sb;
            if (charset == null)
                sb = str.getBytes();
            else
                sb = str.getBytes(charset);
            int len = sb.length;
            if (len > 65535) len = 65535;
            return len + 2;
        }
    }

    public int writeSPasString0(String str, String charset)
            throws IOException {
        if (str == null) {
            writeShort(0);
            write(0);
            return 1;
        } else {
            byte[] sb;
            if (charset == null)
                sb = str.getBytes();
            else
                sb = str.getBytes(charset);
            int len = sb.length;
            if (len > 65535) len = 65535;
            writeShort(len);
            write(sb, 0, len);
            write(0);
            return len + 1;
        }
    }

    public int calcSPasString0(String str, String charset)
            throws IOException {
        if (str == null) {
            return 3;
        } else {
            byte[] sb;
            if (charset == null)
                sb = str.getBytes();
            else
                sb = str.getBytes(charset);
            int len = sb.length;
            if (len > 65535) len = 65535;
            return len + 3;
        }
    }
}
