/**
 *     Caption: Zaval Advanced TV Parser
 *     $Revision: 1.1.1.1 $
 *     $Date: 2003/09/10 13:01:02 $
 *
 *     @author:     Alex Buloichik
 *     @version:    1.0
 *
 * The Zaval Advanced TV Parser (also known as ATV-Parser) is a part of the
 * Zaval Advanced TV Guide package that allows to download/view/convert
 * programs listings.
 *
 * For more info on this product read Zaval Advanced TV Guide User's Guide
 * (It comes within this package).
 * The latest product version is always available from the product's homepage:
 * http://www.zaval.org/products/atv/
 * and from the SourceForge:
 * http://sourceforge.net/projects/atv/
 *
 * Contacts:
 *   Support : support@zaval.org
 *   Change Requests : change-request@zaval.org
 *   Feedback : feedback@zaval.org
 *   Other : info@zaval.org
 *
 * Copyright (C) 2001-2002  Zaval Creative Engineering Group (http://www.zaval.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.zaval.util.io;

import java.util.*;
import java.util.regex.*;
import java.io.*;

public class AProperties extends HashMap {
    public void load(String filename)
            throws FileNotFoundException, IOException {
        try {
            Pattern reProp = Pattern.compile("^(.+)=(.+)$");
            Matcher m;
            BufferedReader inpr = new BufferedReader(new InputStreamReader(new FileInputStream(filename)));
            String str;
            while ((str = inpr.readLine()) != null) {
                str = str.trim();
                int b = str.indexOf("#");
                if (b >= 0) str = str.substring(0, b);
                str = str.trim();
                if ((m = reProp.matcher(str)).find()) put(m.group(1), m.group(2));
            }
            inpr.close();
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE in AProperties : " + e.getMessage());
            System.exit(1);
        }
    }

    public void loadPatch(String filename)
            throws FileNotFoundException, IOException {
        try {
            Pattern reProp = Pattern.compile("^(.+)===(.+)$");
            Matcher m;
            BufferedReader inpr = new BufferedReader(new InputStreamReader(new FileInputStream(filename), "UTF-8"));
            String str;
            while ((str = inpr.readLine()) != null) {
                if ((m = reProp.matcher(str)).find())
                    put(m.group(1), m.group(2));
            }
            inpr.close();
        } catch (PatternSyntaxException e) {
            System.err.println("Error create RE in AProperties : " + e.getMessage());
            System.exit(1);
        }
    }

    public String getProperty(String key, String defval) {
        String res = (String) get(key);
        if (res == null)
            res = defval;
        return res;
    }

    public String getProperty(String key) {
        return (String) get(key);
    }
}
